@@ -1,21 +1,18 @@
 import dataclasses
 import zlib
 from copy import deepcopy
-from typing import Any, Callable, ClassVar, List, Optional, Set, Tuple, Union, overload
+from typing import Any, Callable, ClassVar, List, Literal, Optional, Set, Tuple, TypeVar, Union, overload
+
+from typing_extensions import ParamSpec
 
 from chalk._monitoring.charts_codegen import Series
 from chalk._monitoring.charts_enums_codegen import (
     AlertSeverityKind,
     ChartLinkKind,
-    ComparatorKind,
-    FilterKind,
-    GroupByKind,
     MetricFormulaKind,
-    MetricKind,
     ThresholdPosition,
-    WindowFunctionKind,
 )
-from chalk._monitoring.charts_series_base import MetricFilter, SeriesBase, ThresholdFunction, TSeries
+from chalk._monitoring.charts_series_base import SeriesBase, ThresholdFunction
 from chalk.features import FeatureWrapper
 from chalk.features.resolver import Resolver, ResolverProtocol
 from chalk.utils.duration import parse_chalk_duration
@@ -87,20 +84,27 @@ class _Formula:
 
 class Trigger:
     """
-    Class to attach an alert to a Chart. Usually instantiated with the Chart.with_trigger() method.
+    Class to attach an alert to a Chart. Usually instantiated with the `Chart(...).with_trigger(Trigger(...))` method.
     """
 
     def __init__(
         self,
         name: str,
-        severity: Optional[Union[AlertSeverityKind, str]] = None,
+        severity: Optional[
+            Literal[
+                "CRITICAL",
+                "ERROR",
+                "WARNING",
+                "INFO",
+            ]
+        ] = None,
         threshold_position: Optional[Union[ThresholdPosition, str]] = None,
         threshold_value: Optional[float] = None,
         series_name: Optional[str] = None,
         channel_name: Optional[str] = None,
     ):
         self._name = name
-        self._severity = severity
+        self._severity = severity and AlertSeverityKind(severity.upper())
         self._threshold_position = threshold_position
         self._threshold_value = threshold_value
         self._series_name = series_name
@@ -111,7 +115,15 @@ class Trigger:
         copy._name = name
         return copy
 
-    def with_severity(self, severity: Union[AlertSeverityKind, str]) -> "Trigger":
+    def with_severity(
+        self,
+        severity: Literal[
+            "CRITICAL",
+            "ERROR",
+            "WARNING",
+            "INFO",
+        ],
+    ) -> "Trigger":
         copy = self._copy_with()
         copy._severity = AlertSeverityKind(severity.upper())
         return copy
@@ -178,7 +190,11 @@ class Trigger:
         return zlib.crc32(trigger_string.encode())
 
 
-def _copy_with(function: Callable):
+P = ParamSpec("P")
+T = TypeVar("T")
+
+
+def _copy_with(function: Callable[P, "Chart"]) -> Callable[P, "Chart"]:
     def inner(self, *args, **kwargs):
         copy = deepcopy(self)
         if not self._keep:
@@ -199,7 +215,7 @@ class Chart:
 
     _registry: ClassVar[Set[Union[str, "Chart"]]] = set()
 
-    def __init__(self, name: str, window_period: Optional[str] = None, keep: Optional[bool] = False):
+    def __init__(self, name: str, window_period: str = "1h", keep: Optional[bool] = False):
         """
 
         Parameters
@@ -230,7 +246,7 @@ class Chart:
 
     @_copy_with
     def with_name(self, name: str) -> "Chart":
-        """
+        """Override the name of a chart.
 
         Parameters
         ----------
@@ -247,7 +263,7 @@ class Chart:
 
     @_copy_with
     def with_window_period(self, window_period: str) -> "Chart":
-        """
+        """Change the window period for a chart.
 
         Parameters
         ----------
@@ -263,171 +279,9 @@ class Chart:
         self._window_period = window_period
         return self
 
-    # def with_feature_request_count_series(
-    #     self, series_name: str, window_function: Union[WindowFunctionKind, str], time_shift: Optional[int] = None
-    # ) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name,
-    #         metric=MetricKind.FEATURE_REQUEST_COUNT,
-    #         window_function=window_function,
-    #         time_shift=time_shift,
-    #     )
-
-    # def with_feature_latency_series(
-    #     self, series_name: str, window_function: Union[WindowFunctionKind, str], time_shift: Optional[int] = None
-    # ) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name, metric=MetricKind.FEATURE_LATENCY, window_function=window_function, time_shift=time_shift
-    #     )
-
-    # def with_feature_staleness_series(
-    #     self, series_name: str, window_function: Union[WindowFunctionKind, str], time_shift: Optional[int] = None
-    # ) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name,
-    #         metric=MetricKind.FEATURE_STALENESS,
-    #         window_function=window_function,
-    #         time_shift=time_shift,
-    #     )
-
-    # def with_feature_value_series(
-    #     self, series_name: str, window_function: Union[WindowFunctionKind, str], time_shift: Optional[int] = None
-    # ) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name, metric=MetricKind.FEATURE_VALUE, window_function=window_function, time_shift=time_shift
-    #     )
-
-    # def with_feature_write_series(
-    #     self, series_name: str, window_function: Union[WindowFunctionKind, str], time_shift: Optional[int] = None
-    # ) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name, metric=MetricKind.FEATURE_WRITE, window_function=window_function, time_shift=time_shift
-    #     )
-
-    # def with_feature_null_ratio_series(self, series_name: str, time_shift: Optional[int] = None) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name,
-    #         metric=MetricKind.FEATURE_NULL_RATIO,
-    #         time_shift=time_shift,
-    #     )
-
-    # def with_resolver_request_count_series(
-    #     self, series_name: str, window_function: Union[WindowFunctionKind, str], time_shift: Optional[int] = None
-    # ) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name,
-    #         metric=MetricKind.RESOLVER_REQUEST_COUNT,
-    #         window_function=window_function,
-    #         time_shift=time_shift,
-    #     )
-
-    # def with_resolver_latency_series(
-    #     self, series_name: str, window_function: Union[WindowFunctionKind, str], time_shift: Optional[int] = None
-    # ) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name, metric=MetricKind.RESOLVER_LATENCY, window_function=window_function, time_shift=time_shift
-    #     )
-
-    # def with_resolver_success_ratio_series(self, series_name: str, time_shift: Optional[int] = None) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name,
-    #         metric=MetricKind.RESOLVER_SUCCESS_RATIO,
-    #         time_shift=time_shift,
-    #     )
-
-    # def with_query_count_series(
-    #     self, series_name: str, window_function: Union[WindowFunctionKind, str], time_shift: Optional[int] = None
-    # ) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name, metric=MetricKind.QUERY_COUNT, window_function=window_function, time_shift=time_shift
-    #     )
-
-    # def with_query_latency_series(
-    #     self, series_name: str, window_function: Union[WindowFunctionKind, str], time_shift: Optional[int] = None
-    # ) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name, metric=MetricKind.QUERY_LATENCY, window_function=window_function, time_shift=time_shift
-    #     )
-
-    # def with_query_success_ratio_series(self, series_name: str, time_shift: Optional[int] = None) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name,
-    #         metric=MetricKind.QUERY_SUCCESS_RATIO,
-    #         time_shift=time_shift,
-    #     )
-
-    # def with_billing_inference_series(self, series_name: str, time_shift: Optional[int] = None) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name,
-    #         metric=MetricKind.BILLING_INFERENCE,
-    #         time_shift=time_shift,
-    #     )
-
-    # def with_billing_cron_series(self, series_name: str, time_shift: Optional[int] = None) -> "Chart":
-    #     return self.with_series(name=series_name, metric=MetricKind.BILLING_CRON, time_shift=time_shift)
-
-    # def with_billing_migration_series(self, series_name: str, time_shift: Optional[int] = None) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name,
-    #         metric=MetricKind.BILLING_MIGRATION,
-    #         time_shift=time_shift,
-    #     )
-
-    # def with_cron_count_series(self, series_name: str, time_shift: Optional[int] = None) -> "Chart":
-    #     return self.with_series(name=series_name, metric=MetricKind.CRON_COUNT, time_shift=time_shift)
-
-    # def with_cron_latency_series(
-    #     self, series_name: str, window_function: Union[WindowFunctionKind, str], time_shift: Optional[int] = None
-    # ) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name, metric=MetricKind.CRON_LATENCY, window_function=window_function, time_shift=time_shift
-    #     )
-
-    # def with_stream_messages_processed_series(
-    #     self,
-    #     series_name: str,
-    #     window_function: Union[WindowFunctionKind, str],
-    #     time_shift: Optional[int] = None,
-    # ) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name,
-    #         metric=MetricKind.STREAM_MESSAGES_PROCESSED,
-    #         window_function=window_function,
-    #         time_shift=time_shift,
-    #     )
-
-    # def with_stream_message_latency_series(
-    #     self, series_name: str, window_function: Union[WindowFunctionKind, str], time_shift: Optional[int] = None
-    # ) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name,
-    #         metric=MetricKind.STREAM_MESSAGE_LATENCY,
-    #         window_function=window_function,
-    #         time_shift=time_shift,
-    #     )
-
-    # def with_stream_windows_processed_series(
-    #     self, series_name: str, window_function: Union[WindowFunctionKind, str], time_shift: Optional[int] = None
-    # ) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name,
-    #         metric=MetricKind.STREAM_WINDOWS_PROCESSED,
-    #         window_function=window_function,
-    #         time_shift=time_shift,
-    #     )
-
-    # def with_stream_window_latency_series(
-    #     self, series_name: str, window_function: Union[WindowFunctionKind, str], time_shift: Optional[int] = None
-    # ) -> "Chart":
-    #     return self.with_series(
-    #         name=series_name,
-    #         metric=MetricKind.STREAM_WINDOW_LATENCY,
-    #         window_function=window_function,
-    #         time_shift=time_shift,
-    #     )
     @_copy_with
-    def with_series(self, series: TSeries) -> "Chart":
-        """Attaches a Series to your Chart instance.
+    def with_series(self, series: SeriesBase) -> "Chart":
+        """Attaches a `Series` to your `Chart` instance.
 
         Parameters
         ----------
@@ -441,7 +295,7 @@ class Chart:
             A copy of your chart with the new name
         """
         if not isinstance(series, SeriesBase):
-            raise ValueError(f"'series' value '{series.name}' must be a Series object")
+            raise ValueError(f"'series' value '{series}' must be a Series object")
         self._series.append(series)
         if series._entity_id:
             self._entity_id = series._entity_id
@@ -449,7 +303,8 @@ class Chart:
         return self
 
     def get_series(self, series_name: str) -> SeriesBase:
-        """Get a series from your chart by series name.
+        """Get a `Series` from your `Chart` by series name.
+
         It is advised to use different series names within your charts.
 
         Parameters
@@ -460,7 +315,7 @@ class Chart:
         Returns
         -------
         Series
-            The first series added to your Chart with the given series name.
+            The first series added to your `Chart` with the given series name.
         """
         for series in self._series:
             if series._name == series_name:
@@ -504,8 +359,7 @@ class Chart:
         severity: Optional[Union[AlertSeverityKind, str]] = None,
         channel_name: Optional[str] = None,
     ) -> "Chart":
-        """Attaches a trigger to your Chart.
-        Your Chart may optionally have one Trigger.
+        """Attaches a `Trigger` to your `Chart`. Your `Chart` may optionally have one `Trigger`.
 
         Parameters
         ----------
@@ -550,7 +404,7 @@ class Chart:
     @_copy_with
     def with_feature_link(self, feature: Any) -> "Chart":
         """Explicitly link a Chart to a feature.
-        This Chart will then be visible on the webpage for this feature.
+        This chart will then be visible on the webpage for this feature.
         Charts may only be linked to one entity.
 
         Parameters
@@ -569,8 +423,8 @@ class Chart:
 
     @_copy_with
     def with_resolver_link(self, resolver: ResolverProtocol) -> "Chart":
-        """Explicitly link a Chart to a resolver.
-        This Chart will then be visible on the webpage for this resolver.
+        """Explicitly link a chart to a resolver.
+        This chart will then be visible on the webpage for this resolver.
         Charts may only be linked to one entity.
 
         Parameters
@@ -589,8 +443,8 @@ class Chart:
 
     @_copy_with
     def with_query_link(self, query_name: str) -> "Chart":
-        """Explicitly link a Chart to a query.
-        This Chart will then be visible on the webpage for this query.
+        """Explicitly link a chart to a query.
+        This chart will then be visible on the webpage for this query.
         Charts may only be linked to one entity.
 
         Parameters
@@ -608,11 +462,12 @@ class Chart:
         return self
 
     def keep(self) -> "Chart":
-        """Designates that this Chart and all of its descendants will be registered.
+        """Designates that this chart and all of its descendants will be registered.
 
         Returns
         -------
         Chart
+            The same chart.
         """
         self._keep = True
         return self
@@ -621,6 +476,18 @@ class Chart:
         return f"Chart(name='{self._name}')"
 
     def __getitem__(self, key: str) -> Union[SeriesBase, _Formula]:
+        """Retrieve a series or formula by name from a chart
+
+        Parameters
+        ----------
+        key
+            The name of the series or formula to retrieve.
+
+        Returns
+        -------
+        The series or formula with the given name.
+
+        """
         for series in self._series:
             if series._name == key:
                 return series
