@@ -3,21 +3,23 @@ from typing import Union
 from chalk._monitoring.Chart import (
     AlertSeverityKind,
     Chart,
-    ComparatorKind,
-    FilterKind,
-    GroupByKind,
-    MetricFilter,
     MetricFormulaKind,
-    MetricKind,
     Series,
     ThresholdPosition,
     Trigger,
-    WindowFunctionKind,
     _DatasetFeatureOperand,
     _Formula,
     _MultiSeriesOperand,
     _SingleSeriesOperand,
 )
+from chalk._monitoring.charts_enums_codegen import (
+    ComparatorKind,
+    FilterKind,
+    GroupByKind,
+    MetricKind,
+    WindowFunctionKind,
+)
+from chalk._monitoring.charts_series_base import MetricFilter
 from chalk.parsed.duplicate_input_gql import (
     AlertSeverityKindGQL,
     ComparatorKindGQL,
