@@ -29,6 +29,7 @@ from chalk.client.models import (
     WhoAmIResponse,
 )
 from chalk.features import DataFrame, Feature
+from chalk.features.tag import BranchId, EnvironmentId
 
 if TYPE_CHECKING:
     import polars as pl
@@ -46,7 +47,7 @@ class DatasetRevision(Protocol):
     outputs: List[str]
     """Output features for the dataset revision."""
 
-    givens_uri: str
+    givens_uri: str | None
     """Location of the givens stored for the dataset."""
 
     status: QueryStatus
@@ -87,8 +88,6 @@ class DatasetRevision(Protocol):
 
     def get_data_as_polars(
         self,
-        context: Optional[OfflineQueryContext] = None,
-        preview_deployment_id: Optional[str] = None,
         output_id: bool = False,
         output_ts: bool = False,
     ) -> pl.LazyFrame:
@@ -96,10 +95,6 @@ class DatasetRevision(Protocol):
 
         Other Parameters
         ----------
-        context
-            The environment under which you'd like to query your data.
-        preview_deployment_id
-            If specified, Chalk will route your request to the relevant preview deployment.
         output_id
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `pl.LazyFrame`.
@@ -119,8 +114,6 @@ class DatasetRevision(Protocol):
 
     def get_data_as_pandas(
         self,
-        context: Optional[OfflineQueryContext] = None,
-        preview_deployment_id: Optional[str] = None,
         output_id: bool = False,
         output_ts: bool = False,
     ) -> pd.DataFrame:
@@ -128,10 +121,6 @@ class DatasetRevision(Protocol):
 
         Other Parameters
         ----------
-        context
-            The environment under which you'd like to query your data.
-        preview_deployment_id
-            If specified, Chalk will route your request to the relevant preview deployment.
         output_id
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `pd.DataFrame`.
@@ -151,8 +140,6 @@ class DatasetRevision(Protocol):
 
     def get_data_as_dataframe(
         self,
-        context: Optional[OfflineQueryContext] = None,
-        preview_deployment_id: Optional[str] = None,
         output_id: bool = False,
         output_ts: bool = False,
     ) -> DataFrame:
@@ -161,10 +148,6 @@ class DatasetRevision(Protocol):
 
         Other Parameters
         ----------
-        context
-            The environment under which you'd like to query your data.
-        preview_deployment_id
-            If specified, Chalk will route your request to the relevant preview deployment.
         output_id
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `DataFrame`.
@@ -182,25 +165,31 @@ class DatasetRevision(Protocol):
     def download_data(
         self,
         path: str,
+        output_id: bool = False,
+        output_ts: bool = False,
         num_executors: Optional[int] = None,
-        context: Optional[OfflineQueryContext] = None,
-        preview_deployment_id: Optional[str] = None,
-    ):
+    ) -> None:
         """Downloads output files pertaining to the revision to given path.
 
+        Datasets are stored in Chalk as sharded Parquet files. With this
+        method, you can download those raw files into a directory for processing
+        with other tools.
+
         Parameters
         ----------
         path
-            Directory to download files to.
+            A directory where the Parquet files from the dataset will be downloaded.
 
         Other Parameters
         ----------
+        output_id
+            Whether to return the primary key feature in a column
+            named `"__chalk__.__id__"` in the resulting `DataFrame`.
+        output_ts
+            Whether to return the timestamp feature in a column
+            named `__chalk__.CHALK_TS` in the resulting `DataFrame`.
         num_executors
             Number of executors to use to download files in parallel.
-        context
-            The environment under which you'd like to query your data.
-        preview_deployment_id
-            If specified, Chalk will route your request to the relevant preview deployment.
         """
         ...
 
@@ -253,8 +242,6 @@ class Dataset(Protocol):
 
     def get_data_as_polars(
         self,
-        context: Optional[OfflineQueryContext] = None,
-        preview_deployment_id: Optional[str] = None,
         output_id: bool = False,
         output_ts: bool = False,
     ) -> pl.LazyFrame:
@@ -262,10 +249,6 @@ class Dataset(Protocol):
 
         Other Parameters
         ----------
-        context
-            The environment under which you'd like to query your data.
-        preview_deployment_id
-            If specified, Chalk will route your request to the relevant preview deployment.
         output_id
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `pl.LazyFrame`.
@@ -285,8 +268,6 @@ class Dataset(Protocol):
 
     def get_data_as_pandas(
         self,
-        context: Optional[OfflineQueryContext] = None,
-        preview_deployment_id: Optional[str] = None,
         output_id: bool = False,
         output_ts: bool = False,
     ) -> pd.DataFrame:
@@ -294,10 +275,6 @@ class Dataset(Protocol):
 
         Other Parameters
         ----------
-        context
-            The environment under which you'd like to query your data.
-        preview_deployment_id
-            If specified, Chalk will route your request to the relevant preview deployment.
         output_id
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `pd.DataFrame`.
@@ -317,20 +294,13 @@ class Dataset(Protocol):
 
     def get_data_as_dataframe(
         self,
-        context: Optional[OfflineQueryContext] = None,
-        preview_deployment_id: Optional[str] = None,
         output_id: bool = False,
         output_ts: bool = False,
     ) -> DataFrame:
         """Loads a Chalk `DataFrame` containing the output of the most recent revision.
 
-
         Other Parameters
         ----------
-        context
-            The environment under which you'd like to query your data.
-        preview_deployment_id
-            If specified, Chalk will route your request to the relevant preview deployment.
         output_id
             Whether to return the primary key feature in a column
             named `"__chalk__.__id__"` in the resulting `DataFrame`.
@@ -341,7 +311,7 @@ class Dataset(Protocol):
         Returns
         -------
         DataFrame
-            a `DataFrame` materializing query output data.
+            A `DataFrame` materializing query output data.
         """
         ...
 
@@ -349,24 +319,22 @@ class Dataset(Protocol):
         self,
         path: str,
         num_executors: Optional[int] = None,
-        context: Optional[OfflineQueryContext] = None,
-        preview_deployment_id: Optional[str] = None,
     ):
-        """Downloads output files of the most recent revision to given path.
+        """Downloads output files pertaining to the revision to given path.
+
+        Datasets are stored in Chalk as sharded Parquet files. With this
+        method, you can download those raw files into a directory for processing
+        with other tools.
 
         Parameters
         ----------
         path
-            Directory to download files to.
+            A directory where the Parquet files from the dataset will be downloaded.
 
         Other Parameters
         ----------
         num_executors
             Number of executors to use to download files in parallel.
-        context
-            The environment under which you'd like to query your data.
-        preview_deployment_id
-            If specified, Chalk will route your request to the relevant preview deployment.
 
         Examples
         ----------
@@ -487,7 +455,7 @@ class ChalkClient:
         self,
         client_id: str,
         client_secret: str,
-        environment: Optional[str] = None,
+        environment: Optional[EnvironmentId] = None,
         api_server: Optional[str] = None,
     ):
         """Create a `ChalkClient` with the given credentials.
@@ -526,7 +494,7 @@ class ChalkClient:
         self,
         client_id: Optional[str] = None,
         client_secret: Optional[str] = None,
-        environment: Optional[str] = None,
+        environment: Optional[EnvironmentId] = None,
         api_server: Optional[str] = None,
     ):
         """Create a `ChalkClient`. Clients can either be created
@@ -541,8 +509,9 @@ class ChalkClient:
     def trigger_resolver_run(
         self,
         resolver_fqn: str,
-        context: Optional[OfflineQueryContext] = None,
+        environment: Optional[EnvironmentId] = None,
         preview_deployment_id: Optional[str] = None,
+        branch: Optional[BranchId] = None,
     ) -> ResolverRunResponse:
         """Triggers a resolver to run.
         See https://docs.chalk.ai/docs/runs for more information.
@@ -551,11 +520,17 @@ class ChalkClient:
         ----------
         resolver_fqn
             The fully qualified name of the resolver to trigger.
-        context
-            The environment under which you'd like to query your data.
+        environment
+            The environment under which to run the resolvers.
+            API tokens can be scoped to an environment.
+            If no environment is specified in the query,
+            but the token supports only a single environment,
+            then that environment will be taken as the scope
+            for executing the request.
         preview_deployment_id
             If specified, Chalk will route your request to the
             relevant preview deployment.
+        branch
 
         Returns
         -------
@@ -574,7 +549,9 @@ class ChalkClient:
     def get_run_status(
         self,
         run_id: str,
+        environment: Optional[EnvironmentId] = None,
         preview_deployment_id: Optional[str] = None,
+        branch: Optional[BranchId] = None,
     ) -> ResolverRunResponse:
         """Retrieves the status of a resolver run.
         See https://docs.chalk.ai/docs/runs for more information.
@@ -583,9 +560,17 @@ class ChalkClient:
         ----------
         run_id
             ID of the resolver run to check.
+        environment
+            The environment under which to run the resolvers.
+            API tokens can be scoped to an environment.
+            If no environment is specified in the query,
+            but the token supports only a single environment,
+            then that environment will be taken as the scope
+            for executing the request.
         preview_deployment_id
             If specified, Chalk will route your request to the
             relevant preview deployment.
+        branch
 
         Returns
         -------
@@ -613,7 +598,7 @@ class ChalkClient:
         Returns
         -------
         WhoAmIResponse
-            The identity of your client
+            The identity of your client.
 
         Examples
         --------
@@ -628,8 +613,11 @@ class ChalkClient:
         input: Mapping[Union[str, Feature, Any], Any],
         output: Sequence[Union[str, Feature, Any]],
         staleness: Optional[Mapping[Union[str, Feature, Any], str]] = None,
-        context: Optional[OnlineQueryContext] = None,
+        context: Optional[OnlineQueryContext] = None,  # Deprecated.
+        environment: Optional[EnvironmentId] = None,
+        tags: Optional[List[str]] = None,
         preview_deployment_id: Optional[str] = None,
+        branch: Optional[BranchId] = None,
         correlation_id: Optional[str] = None,
         query_name: Optional[str] = None,
         meta: Optional[Mapping[str, str]] = None,
@@ -646,9 +634,18 @@ class ChalkClient:
         staleness
             Maximum staleness overrides for any output features or intermediate features.
             See https://docs.chalk.ai/docs/query-caching for more information.
-        context
-            The context object controls the environment and tags
-            under which a request should execute resolvers.
+        environment
+            The environment under which to run the resolvers.
+            API tokens can be scoped to an environment.
+            If no environment is specified in the query,
+            but the token supports only a single environment,
+            then that environment will be taken as the scope
+            for executing the request.
+        tags
+            The tags used to scope the resolvers.
+            More information at https://docs.chalk.ai/docs/resolver-tags
+        branch
+            If specified, Chalk will route your request to the relevant branch.
         preview_deployment_id
             If specified, Chalk will route your request to the
             relevant preview deployment.
@@ -662,7 +659,9 @@ class ChalkClient:
             available in web interfaces. If None, a correlation ID will be
             generated for you and returned on the response.
         meta
-            Arbitrary key:value pairs to associate with a query.
+            Arbitrary `key:value` pairs to associate with a query.
+        context
+            Deprecated in favor of `environment` and `tags`.
 
         Returns
         -------
@@ -684,7 +683,8 @@ class ChalkClient:
     def upload_features(
         self,
         input: Mapping[Union[str, Feature, Any], Any],
-        context: Optional[OnlineQueryContext] = None,
+        branch: Optional[BranchId] = None,
+        environment: Optional[EnvironmentId] = None,
         preview_deployment_id: Optional[str] = None,
         correlation_id: Optional[str] = None,
         query_name: Optional[str] = None,
@@ -696,9 +696,14 @@ class ChalkClient:
         ----------
         input
             The features for which there are known values, mapped to those values.
-        context
-            The context object controls the environment and tags
-            under which a request should execute resolvers.
+        branch
+        environment
+            The environment under which to run the resolvers.
+            API tokens can be scoped to an environment.
+            If no environment is specified in the query,
+            but the token supports only a single environment,
+            then that environment will be taken as the scope
+            for executing the request.
         preview_deployment_id
             If specified, Chalk will route your request to the relevant preview deployment
         query_name
@@ -736,9 +741,9 @@ class ChalkClient:
         input_times: Union[Sequence[datetime], datetime, None] = None,
         output: Sequence[Union[str, Feature, Any]] = (),
         required_output: Sequence[Union[str, Feature, Any]] = (),
-        context: Optional[OfflineQueryContext] = None,
+        environment: Optional[EnvironmentId] = None,
         dataset_name: Optional[str] = None,
-        branch: Optional[str] = None,
+        branch: Optional[BranchId] = None,
         max_samples: Optional[int] = None,
     ) -> Dataset:
         """Compute feature values from the offline store.
@@ -758,15 +763,21 @@ class ChalkClient:
             The features that you'd like to sample, if they exist.
             If an output feature was never computed for a sample (row) in
             the resulting `DataFrame`, its value will be `None`.
-        context
-            The environment under which you'd like to query your data.
+        environment
+            The environment under which to run the resolvers.
+            API tokens can be scoped to an environment.
+            If no environment is specified in the query,
+            but the token supports only a single environment,
+            then that environment will be taken as the scope
+            for executing the request.
         dataset_name
             A unique name that if provided will be used to generate and
-            save a dataset constructed from the list of features computed
+            save a `Dataset` constructed from the list of features computed
             from the inputs.
         max_samples
             The maximum number of samples to include in the `DataFrame`.
             If not specified, all samples will be returned.
+        branch
 
         Other Parameters
         ----------------
@@ -781,7 +792,7 @@ class ChalkClient:
         Returns
         -------
         Dataset
-            A Chalk 'Dataset'.
+            A Chalk `Dataset`.
 
         Examples
         --------
@@ -816,13 +827,14 @@ class ChalkClient:
         input_times: Union[Sequence[datetime], datetime, None] = None,
         output: Sequence[Union[str, Feature, Any]] = (),
         required_output: Sequence[Union[str, Feature, Any]] = (),
-        output_ts: bool = True,
+        output_ts: bool = False,
         context: Optional[OfflineQueryContext] = None,
         dataset: Optional[str] = None,
-        branch: Optional[str] = None,
         max_samples: Optional[int] = None,
     ) -> pd.DataFrame:
-        """Compute feature values from the offline store.
+        """Deprecated. Use `.offline_query` instead.
+
+        Compute feature values from the offline store.
         See https://docs.chalk.ai/docs/training-client for more information.
 
         Parameters
@@ -898,8 +910,8 @@ class ChalkClient:
         output_ts: bool = False,
         max_samples: Optional[int] = None,
         dataset: Optional[str] = None,
-        branch: Optional[str] = None,
-        context: Optional[OfflineQueryContext] = None,
+        branch: Optional[BranchId] = None,
+        environment: Optional[EnvironmentId] = None,
     ) -> pd.DataFrame:
         """Get the most recent feature values from the offline store.
 
@@ -913,8 +925,16 @@ class ChalkClient:
             in the resulting `DataFrame`, its value will be `None`.
         max_samples
             The maximum number of rows to return.
-        context
-            The environment under which you'd like to query your data.
+        environment
+            The environment under which to run the resolvers.
+            API tokens can be scoped to an environment.
+            If no environment is specified in the query,
+            but the token supports only a single environment,
+            then that environment will be taken as the scope
+            for executing the request.
+        dataset
+            The `Dataset` name under which to save the output.
+        branch
 
         Other Parameters
         ----------------
@@ -955,24 +975,29 @@ class ChalkClient:
     def get_dataset(
         self,
         dataset_name: str,
-        context: Optional[OfflineQueryContext] = None,
+        environment: Optional[EnvironmentId] = None,
     ) -> Dataset:
         """Get a Chalk `Dataset` containing data from a previously created dataset.
 
-        If an offline query has been created with a dataset name, `get_dataset` will
+        If an offline query has been created with a dataset name, `.get_dataset` will
         return a Chalk `Dataset`.
-        The 'Dataset' wraps a lazily-loading Chalk 'DataFrame' that enables us to analyze our data without loading all of it
-        directly into memory.
+        The `Dataset` wraps a lazily-loading Chalk `DataFrame` that enables us to analyze
+        our data without loading all of it directly into memory.
         See https://docs.chalk.ai/docs/training-client for more information.
 
         Parameters
         ----------
         dataset_name
-            The name of the dataset to return.
+            The name of the `Dataset` to return.
             Previously, you must have supplied a dataset name upon an offline query.
             Dataset names are unique for each environment.
-        context
-            The environment under which you'd like to query your data.
+        environment
+            The environment under which to run the resolvers.
+            API tokens can be scoped to an environment.
+            If no environment is specified in the query,
+            but the token supports only a single environment,
+            then that environment will be taken as the scope
+            for executing the request.
 
         Returns
         -------
@@ -1007,11 +1032,11 @@ class ChalkClient:
 
         If memory allows:
 
-        >>> df: pd.DataFrame = dataset.data.to_pandas()
+        >>> df: pd.DataFrame = dataset.data_as_pandas
         """
         ...
 
-    def __new__(cls, *args, **kwargs):
+    def __new__(cls, *args: Any, **kwargs: Any):
         from chalk.client.client_impl import ChalkAPIClientImpl
 
         return ChalkAPIClientImpl(*args, **kwargs)
