@@ -5,7 +5,7 @@ import time
 import uuid
 import warnings
 from datetime import datetime, timezone
-from typing import TYPE_CHECKING, Any, List, Mapping, Optional, Sequence, Type, TypeVar, Union, cast
+from typing import TYPE_CHECKING, Any, List, Literal, Mapping, Optional, Sequence, Type, TypeVar, Union, cast, overload
 from urllib.parse import urljoin
 
 import pandas as pd
@@ -51,6 +51,7 @@ from chalk.client.models import (
 from chalk.config.auth_config import load_token
 from chalk.features import DataFrame, Feature, FeatureNotFoundException, ensure_feature
 from chalk.features.pseudofeatures import CHALK_TS_FEATURE
+from chalk.features.tag import BranchId, EnvironmentId
 from chalk.utils.log_with_context import get_logger
 from chalk.utils.missing_dependency import missing_dependency_exception
 
@@ -70,16 +71,56 @@ class _ChalkClientConfig(BaseModel):
     active_environment: Optional[str]
 
 
-def _normalize_input_times(input_times: Union[datetime, Sequence[datetime], None], input_len: int):
+def _to_offline_query_input(
+    input: Union[Mapping[Union[str, Feature, Any], Any], pd.DataFrame, pl.DataFrame, DataFrame],
+    input_times: Sequence[datetime] | datetime | None,
+) -> OfflineQueryInput:
+    try:
+        import polars as pl
+    except ImportError:
+        raise missing_dependency_exception("chalkpy[runtime]")
+    if isinstance(input, (DataFrame, pl.DataFrame)):
+        input = input.to_pandas()
+    if isinstance(input, collections.abc.Mapping):
+        input = {str(k): v for (k, v) in input.items()}
+    if not isinstance(input, pd.DataFrame):
+        input = pd.DataFrame(input)
+
+    columns = input.columns
+    matrix: List[List[Any]] = input.T.values.tolist()
+
+    columns_fqn = [str(c) for c in (*columns, CHALK_TS_FEATURE)]
     if input_times is None:
         input_times = datetime.now(timezone.utc)
     if isinstance(input_times, datetime):
-        input_times = [input_times for _ in range(input_len)]
+        input_times = [input_times for _ in range(len(input))]
     local_tz = datetime.now(timezone.utc).astimezone().tzinfo
 
     input_times = [x.replace(tzinfo=local_tz) if x.tzinfo is None else x for x in input_times]
     input_times = [x.astimezone(timezone.utc) for x in input_times]
-    return input_times
+
+    matrix.append([a for a in input_times])
+
+    for col_index, column in enumerate(matrix):
+        for row_index, value in enumerate(column):
+            try:
+                f = Feature.from_root_fqn(columns_fqn[col_index])
+            except FeatureNotFoundException:
+                # The feature is not in the graph, so passing the value as-is and hoping it's possible
+                # to json-serialize it
+                encoded_feature = value
+            else:
+                encoded_feature = f.converter.from_rich_to_json(
+                    value,
+                    missing_value_strategy="error",
+                )
+
+            matrix[col_index][row_index] = encoded_feature
+
+    return OfflineQueryInput(
+        columns=columns_fqn,
+        values=matrix,
+    )
 
 
 class OnlineQueryResponseImpl(OnlineQueryResult):
@@ -87,7 +128,12 @@ class OnlineQueryResponseImpl(OnlineQueryResult):
     errors: List[ChalkError]
     warnings: List[str]
 
-    def __init__(self, data: List[FeatureResult], errors: List[ChalkError], warnings: List[str]):
+    def __init__(
+        self,
+        data: List[FeatureResult],
+        errors: List[ChalkError],
+        warnings: List[str],
+    ):
         self.data = data
         self.errors = errors
         self.warnings = warnings
@@ -128,21 +174,22 @@ class OnlineQueryResponseImpl(OnlineQueryResult):
 
 
 class ChalkAPIClientImpl(ChalkClient):
-    def __new__(cls, *args, **kwargs):
-        obj = object.__new__(ChalkAPIClientImpl)
-        obj.__init__(*args, **kwargs)
-        obj.__name__ = "ChalkClient"
-        obj.__qualname__ = "chalk.client.ChalkClient"
-        return obj
+    __name__ = "ChalkClient"
+    __qualname__ = "chalk.client.ChalkClient"
 
     def __repr__(self):
         return f"chalk.client.ChalkClient<{self._config.name}>"
 
+    def __new__(cls, *args: Any, **kwargs: Any) -> ChalkClient:
+        obj = object.__new__(ChalkAPIClientImpl)
+        obj.__init__(*args, **kwargs)
+        return obj
+
     def __init__(
         self,
         client_id: Optional[str] = None,
         client_secret: Optional[str] = None,
-        environment: Optional[str] = None,
+        environment: Optional[EnvironmentId] = None,
         api_server: Optional[str] = None,
         _skip_cache: bool = False,
     ):
@@ -199,13 +246,20 @@ class ChalkAPIClientImpl(ChalkClient):
         self._primary_environment = creds.primary_environment
         self._exchanged_credentials = True
 
-    def _get_headers(self, environment_override: Optional[str], preview_deployment_id: Optional[str]):
+    def _get_headers(
+        self,
+        environment_override: Optional[str],
+        preview_deployment_id: Optional[str],
+        branch: Optional[BranchId],
+    ) -> dict[str, str]:
         x_chalk_env_id = environment_override or self._config.active_environment or self._primary_environment
         headers = dict(self._default_headers)  # shallow copy
         if x_chalk_env_id is not None:
             headers["X-Chalk-Env-Id"] = x_chalk_env_id
         if preview_deployment_id is not None:
             headers["X-Chalk-Preview-Deployment"] = preview_deployment_id
+        if branch is not None:
+            headers["X-Chalk-Branch-Id"] = branch
         return headers
 
     @staticmethod
@@ -216,7 +270,7 @@ class ChalkAPIClientImpl(ChalkClient):
             raise exception_cls(errors=errors)
 
     @staticmethod
-    def _raise_http_error(http_error: HTTPError, exception_cls: Type[ChalkBaseException], **exception_kwargs):
+    def _raise_http_error(http_error: HTTPError, exception_cls: Type[ChalkBaseException], **exception_kwargs: Any):
         detail = None
         try:
             response_json = http_error.response.json()
@@ -244,11 +298,11 @@ class ChalkAPIClientImpl(ChalkClient):
         uri: str,
         response: Type[T],
         json: Optional[BaseModel],
-        use_engine: bool,
         environment_override: Optional[str],
         exception_cls: Type[ChalkBaseException],
         preview_deployment_id: Optional[str],
-        **exception_kwargs,
+        branch: Optional[BranchId],
+        **exception_kwargs: Any,
     ) -> T:
         # Track whether we already exchanged credentials for this request
         exchanged_credentials = False
@@ -261,6 +315,7 @@ class ChalkAPIClientImpl(ChalkClient):
         headers = self._get_headers(
             environment_override=environment_override,
             preview_deployment_id=preview_deployment_id,
+            branch=branch,
         )
         url = urljoin(self._config.api_server, uri)
         json_body = json and json.dict()
@@ -284,16 +339,17 @@ class ChalkAPIClientImpl(ChalkClient):
             uri=f"/v1/who-am-i",
             response=WhoAmIResponse,
             json=None,
-            use_engine=False,
             environment_override=None,
             exception_cls=ChalkWhoAmIException,
             preview_deployment_id=None,
+            branch=None,
         )
 
     def upload_features(
         self,
         input: Mapping[Union[str, Feature, Any], Any],
-        context: Optional[OnlineQueryContext] = None,
+        branch: Optional[BranchId] = None,
+        environment: Optional[EnvironmentId] = None,
         preview_deployment_id: Optional[str] = None,
         correlation_id: Optional[str] = None,
         query_name: Optional[str] = None,
@@ -303,7 +359,7 @@ class ChalkAPIClientImpl(ChalkClient):
             input=input,
             output=list(input.keys()),
             staleness=None,
-            context=context,
+            environment=environment,
             preview_deployment_id=preview_deployment_id,
             correlation_id=correlation_id,
             query_name=query_name,
@@ -315,12 +371,17 @@ class ChalkAPIClientImpl(ChalkClient):
         input: Mapping[Union[str, Feature, Any], Any],
         output: Sequence[Union[str, Feature, Any]],
         staleness: Optional[Mapping[Union[str, Feature, Any], str]] = None,
-        context: Optional[OnlineQueryContext] = None,
+        context: Optional[OnlineQueryContext] = None,  # Deprecated.
+        environment: Optional[EnvironmentId] = None,
+        tags: Optional[List[str]] = None,
         preview_deployment_id: Optional[str] = None,
+        branch: Optional[BranchId] = None,
         correlation_id: Optional[str] = None,
         query_name: Optional[str] = None,
         meta: Optional[Mapping[str, str]] = None,
     ) -> OnlineQueryResponseImpl:
+        environment = environment or (context and context.environment)
+        tags = tags or (context and context.tags)
         warnings: List[str] = []
         encoded_inputs = {}
         for feature, value in input.items():
@@ -341,7 +402,10 @@ class ChalkAPIClientImpl(ChalkClient):
             inputs=encoded_inputs,
             outputs=outputs,
             staleness={} if staleness is None else {ensure_feature(k).root_fqn: v for k, v in staleness.items()},
-            context=context,
+            context=OnlineQueryContext(
+                environment=environment,
+                tags=tags,
+            ),
             deployment_id=preview_deployment_id,
             correlation_id=correlation_id,
             query_name=query_name,
@@ -353,10 +417,10 @@ class ChalkAPIClientImpl(ChalkClient):
             uri="/v1/query/online",
             json=request,
             response=OnlineQueryResponse,
-            use_engine=preview_deployment_id is None,
-            environment_override=context.environment if context else None,
+            environment_override=environment,
             exception_cls=ChalkOnlineQueryException,
             preview_deployment_id=preview_deployment_id,
+            branch=branch,
         )
         return OnlineQueryResponseImpl(data=resp.data, errors=resp.errors or [], warnings=warnings)
 
@@ -366,83 +430,26 @@ class ChalkAPIClientImpl(ChalkClient):
         input_times: Union[Sequence[datetime], datetime, None] = None,
         output: Sequence[Union[str, Feature, Any]] = (),
         required_output: Sequence[Union[str, Feature, Any]] = (),
-        context: Optional[OfflineQueryContext] = None,
+        environment: Optional[EnvironmentId] = None,
         dataset_name: Optional[str] = None,
-        branch: Optional[str] = None,
+        branch: Optional[BranchId] = None,
         max_samples: Optional[int] = None,
-        disallow_stale_results: None = None,  # Backwards compatibility
     ) -> DatasetImpl:
-        if disallow_stale_results is not None:
-            warnings.warn(
-                DeprecationWarning("Argument `disallow_stale_results` is deprecated. Results are always fresh.")
-            )
-        if context is None:
-            context = OfflineQueryContext()
-
         try:
             import polars as pl
         except ImportError:
             raise missing_dependency_exception("chalkpy[runtime]")
+        del pl  # unused
 
         if len(output) == 0 and len(required_output) == 0:
             raise ValueError("Either 'output' or 'required_output' must be specified.")
         optional_output_root_fqns = [str(f) for f in output]
         required_output_root_fqns = [str(f) for f in required_output]
 
-        if not input:
-            # ChalkClient().sample(...)
-            response = self._create_dataset_job(
-                query_input=None,
-                optional_output=optional_output_root_fqns,
-                required_output=required_output_root_fqns,
-                max_samples=max_samples,
-                context=context,
-                dataset_name=dataset_name,
-                branch=branch,
-                disallow_stale_results=disallow_stale_results,
-            )
-            return dataset_from_response(response, self)
-
-        # ChalkClient().get_training_dataframe(...)
-        if isinstance(input, (DataFrame, pl.DataFrame)):
-            input = input.to_pandas()
-        if isinstance(input, collections.abc.Mapping):
-            input = {str(k): v for (k, v) in input.items()}
-        if not isinstance(input, pd.DataFrame):
-            input = pd.DataFrame(input)
-
-        columns = input.columns
-        matrix: List[List[Any]] = input.T.values.tolist()
-
-        columns_fqn = [str(c) for c in (*columns, CHALK_TS_FEATURE)]
-
-        if input_times is None:
-            input_times = datetime.now(timezone.utc)
-        if isinstance(input_times, datetime):
-            input_times = [input_times for _ in range(len(input))]
-
-        matrix.append([a for a in input_times])
-
-        for col_index, column in enumerate(matrix):
-            for row_index, value in enumerate(column):
-                try:
-                    f = Feature.from_root_fqn(columns_fqn[col_index])
-                except FeatureNotFoundException:
-                    # The feature is not in the graph, so passing the value as-is and hoping it's possible
-                    # to json-serialize it
-                    encoded_feature = value
-                else:
-                    encoded_feature = f.converter.from_rich_to_json(
-                        value,
-                        missing_value_strategy="error",
-                    )
-
-                matrix[col_index][row_index] = encoded_feature
-
-        query_input = OfflineQueryInput(
-            columns=columns_fqn,
-            values=matrix,
-        )
+        if input is None:
+            query_input = None
+        else:
+            query_input = _to_offline_query_input(input, input_times)
 
         response = self._create_dataset_job(
             optional_output=optional_output_root_fqns,
@@ -450,29 +457,30 @@ class ChalkAPIClientImpl(ChalkClient):
             query_input=query_input,
             dataset_name=dataset_name,
             branch=branch,
-            context=context,
+            context=OfflineQueryContext(environment=environment),
             max_samples=max_samples,
-            disallow_stale_results=disallow_stale_results,
         )
         return dataset_from_response(response, self)
 
     def get_training_dataframe(
         self,
-        input: Union[Mapping[Union[str, Feature, Any], Any], pd.DataFrame, pl.DataFrame, DataFrame],
+        input: Union[
+            Mapping[Union[str, Feature], Sequence[Any]],
+            pd.DataFrame,
+            pl.DataFrame,
+            DataFrame,
+        ],
         input_times: Union[Sequence[datetime], datetime, None] = None,
         output: Sequence[Union[str, Feature, Any]] = (),
         required_output: Sequence[Union[str, Feature, Any]] = (),
         output_ts: bool = False,
         context: Optional[OfflineQueryContext] = None,
         dataset: Optional[str] = None,
-        branch: Optional[str] = None,
         max_samples: Optional[int] = None,
-        disallow_stale_results: None = None,
     ) -> pd.DataFrame:
-        if disallow_stale_results is not None:
-            warnings.warn(
-                DeprecationWarning("Argument `disallow_stale_results` is deprecated. Results are always fresh.")
-            )
+        warnings.warn(
+            DeprecationWarning("Method get_training_dataframe is deprecated. Please consider `.offline_query` instead.")
+        )
         if context is None:
             context = OfflineQueryContext()
         try:
@@ -491,75 +499,16 @@ class ChalkAPIClientImpl(ChalkClient):
         if len(output) == 0 and len(required_output) == 0:
             raise ValueError("Either 'output' or 'required_output' must be specified.")
 
-        return self._get_training_dataframe(
-            input=input,
-            input_times=input_times,
-            optional_output=[str(x) for x in output],
-            required_output=[str(x) for x in required_output],
-            output_id=False,
-            output_ts=output_ts,
-            context=context,
-            dataset=dataset,
-            branch=branch,
-            max_samples=max_samples,
-        )
-
-    def _get_training_dataframe(
-        self,
-        input: pd.DataFrame,
-        input_times: Union[Sequence[datetime], datetime, None],
-        optional_output: List[str],
-        required_output: List[str],
-        output_id: bool,
-        output_ts: bool,
-        max_samples: Optional[int],
-        context: OfflineQueryContext,
-        dataset: Optional[str],
-        branch: Optional[str],
-    ) -> pd.DataFrame:
-        try:
-            import polars as pl
-        except ImportError:
-            raise missing_dependency_exception("chalkpy[runtime]")
-
-        columns = input.columns
-        matrix: List[List[Any]] = input.T.values.tolist()
-
-        columns_fqn = [str(c) for c in (*columns, CHALK_TS_FEATURE)]
-
-        input_times = _normalize_input_times(input_times, len(input))
-
-        matrix.append([a for a in input_times])
-
-        for col_index, column in enumerate(matrix):
-            for row_index, value in enumerate(column):
-                try:
-                    f = Feature.from_root_fqn(columns_fqn[col_index])
-                except FeatureNotFoundException:
-                    # The feature is not in the graph, so passing the value as-is and hoping it's possible
-                    # to json-serialize it
-                    encoded_feature = value
-                else:
-                    encoded_feature = f.converter.from_rich_to_json(
-                        value,
-                        missing_value_strategy="error",
-                    )
-
-                matrix[col_index][row_index] = encoded_feature
-
-        query_input = OfflineQueryInput(
-            columns=columns_fqn,
-            values=matrix,
-        )
+        query_input = _to_offline_query_input(input, input_times)
 
         response = self._create_and_await_offline_query_job(
-            optional_output=optional_output,
-            required_output=required_output,
+            optional_output=[str(x) for x in output],
+            required_output=[str(x) for x in required_output],
             query_input=query_input,
-            output_id=output_id,
+            output_id=False,
             output_ts=output_ts,
             dataset_name=dataset,
-            branch=branch,
+            branch=None,
             context=context,
             max_samples=max_samples,
             preview_deployment_id=None,
@@ -577,20 +526,14 @@ class ChalkAPIClientImpl(ChalkClient):
         output_ts: bool = False,
         max_samples: Optional[int] = None,
         dataset: Optional[str] = None,
-        branch: Optional[str] = None,
-        context: Optional[OfflineQueryContext] = None,
-        disallow_stale_results: None = None,
+        branch: Optional[BranchId] = None,
+        environment: Optional[EnvironmentId] = None,
     ) -> pd.DataFrame:
         try:
             import polars as pl
         except ImportError:
             raise missing_dependency_exception("chalkpy[runtime]")
-        if disallow_stale_results is not None:
-            warnings.warn(
-                DeprecationWarning("Argument `disallow_stale_results` is deprecated. Results are always fresh.")
-            )
-        if context is None:
-            context = OfflineQueryContext()
+        context = OfflineQueryContext(environment=environment)
         optional_output_root_fqns = [str(f) for f in output]
         required_output_root_fqns = [str(f) for f in required_output]
 
@@ -618,63 +561,54 @@ class ChalkAPIClientImpl(ChalkClient):
     def get_dataset(
         self,
         dataset_name: str,
-        context: Optional[OfflineQueryContext] = None,
-        output_ts: bool = False,
-        output_id: bool = False,
+        environment: Optional[EnvironmentId] = None,
     ) -> DatasetImpl:
-        if context is None:
-            context = OfflineQueryContext()
         response: DatasetResponse = self._get_dataset(
             dataset_name=dataset_name,
-            context=context,
-            preview_deployment_id=None,
+            environment=environment,
         )
         if response.errors:
-            raise ChalkDatasetDownloadException(dataset_name=dataset_name, errors=response.errors)
+            raise ChalkDatasetDownloadException(
+                dataset_name=dataset_name,
+                errors=response.errors,
+            )
         return dataset_from_response(response, self)
 
     def trigger_resolver_run(
         self,
         resolver_fqn: str,
-        context: Optional[OfflineQueryContext] = None,
+        environment: Optional[EnvironmentId] = None,
         preview_deployment_id: Optional[str] = None,
+        branch: Optional[BranchId] = None,
     ) -> ResolverRunResponse:
         _logger.debug(f"Triggering resolver {resolver_fqn} to run")
-        if context is None:
-            context = OfflineQueryContext()
-
-        request = _TriggerResolverRunRequest(resolver_fqn=resolver_fqn)
-        response = self._request(
+        return self._request(
             method="POST",
             uri="/v1/runs/trigger",
-            json=request,
+            json=_TriggerResolverRunRequest(resolver_fqn=resolver_fqn),
             response=ResolverRunResponse,
-            use_engine=False,
-            environment_override=context.environment,
+            environment_override=environment,
             exception_cls=ChalkResolverRunException,
             preview_deployment_id=preview_deployment_id,
+            branch=branch,
         )
 
-        return response
-
     def get_run_status(
         self,
         run_id: str,
+        environment: Optional[EnvironmentId] = None,
         preview_deployment_id: Optional[str] = None,
-        context: Optional[OfflineQueryContext] = None,
+        branch: Optional[BranchId] = None,
     ) -> ResolverRunResponse:
-        if context is None:
-            context = OfflineQueryContext()
-
         response = self._request(
             method="GET",
             uri=f"/v1/runs/{run_id}",
             response=ResolverRunResponse,
             json=None,
-            use_engine=False,
-            environment_override=context.environment,
+            environment_override=environment,
             exception_cls=ChalkResolverRunException,
             preview_deployment_id=preview_deployment_id,
+            branch=branch,
         )
 
         return response
@@ -686,7 +620,7 @@ class ChalkAPIClientImpl(ChalkClient):
         query_input: Optional[OfflineQueryInput],
         max_samples: Optional[int],
         dataset_name: Optional[str],
-        branch: Optional[str],
+        branch: Optional[BranchId],
         context: OfflineQueryContext,
         output_id: bool,
         output_ts: bool,
@@ -705,7 +639,8 @@ class ChalkAPIClientImpl(ChalkClient):
         response = self._create_offline_query_job(
             request=req,
             context=context,
-            preview_deployment_id=None,
+            preview_deployment_id=preview_deployment_id,
+            branch=branch,
         )
         self._raise_if_200_with_errors(response=response, exception_cls=ChalkOfflineQueryException)
         return self._await_offline_query_job(
@@ -713,27 +648,50 @@ class ChalkAPIClientImpl(ChalkClient):
             outputs=[*optional_output, *required_output],
             lazy=lazy,
             context=context,
-            preview_deployment_id=preview_deployment_id,
             output_id=output_id,
             output_ts=output_ts,
         )
 
+    @overload
+    def _await_offline_query_job(
+        self,
+        job_id: uuid.UUID,
+        outputs: List[str],
+        lazy: bool,
+        context: Optional[OfflineQueryContext],
+        output_id: bool,
+        output_ts: bool,
+        urls_only: Literal[False] = ...,
+    ) -> Union[pl.DataFrame, pl.LazyFrame]:
+        ...
+
+    @overload
+    def _await_offline_query_job(
+        self,
+        job_id: uuid.UUID,
+        outputs: List[str],
+        lazy: bool,
+        context: Optional[OfflineQueryContext],
+        output_id: bool,
+        output_ts: bool,
+        urls_only: Literal[True],
+    ) -> List[str]:
+        ...
+
     def _await_offline_query_job(
         self,
         job_id: uuid.UUID,
         outputs: List[str],
         lazy: bool,
-        context: Optional[OfflineQueryContext] = None,
-        preview_deployment_id: Optional[str] = None,
-        output_id: Optional[bool] = None,
-        output_ts: Optional[bool] = None,
-        urls_only: Optional[bool] = False,
+        context: Optional[OfflineQueryContext],
+        output_id: bool,
+        output_ts: bool,
+        urls_only: bool = False,
     ) -> Union[pl.DataFrame, pl.LazyFrame, List[str]]:
         while True:
             status = self._get_job_status(
                 job_id=job_id,
-                context=context,
-                preview_deployment_id=preview_deployment_id,
+                environment=context and context.environment,
             )
             if status.is_finished:
                 break
@@ -757,9 +715,8 @@ class ChalkAPIClientImpl(ChalkClient):
         query_input: Optional[OfflineQueryInput],
         max_samples: Optional[int],
         dataset_name: Optional[str],
-        branch: Optional[str],
+        branch: Optional[BranchId],
         context: OfflineQueryContext,
-        disallow_stale_results: bool = None,
     ) -> DatasetResponse:
         req = CreateOfflineQueryJobRequest(
             output=optional_output,
@@ -769,12 +726,12 @@ class ChalkAPIClientImpl(ChalkClient):
             max_samples=max_samples,
             dataset_name=dataset_name,
             branch=branch,
-            max_cache_age_secs=0 if disallow_stale_results else None,
         )
         response = self._create_dataset_request(
             request=req,
             context=context,
             preview_deployment_id=None,
+            branch=branch,
         )
         self._raise_if_200_with_errors(response=response, exception_cls=ChalkOfflineQueryException)
         return response
@@ -786,6 +743,7 @@ class ChalkAPIClientImpl(ChalkClient):
         resolver: str,
         context: Optional[OfflineQueryContext] = None,
         preview_deployment_id: Optional[str] = None,
+        branch: Optional[BranchId] = None,
     ) -> pl.DataFrame:
         try:
             import polars as pl
@@ -793,35 +751,7 @@ class ChalkAPIClientImpl(ChalkClient):
             raise missing_dependency_exception("chalkpy[runtime]")
         if context is None:
             context = OfflineQueryContext()
-        if not isinstance(input, DataFrame):
-            input = DataFrame(input)
-        input = input.to_pandas()
-
-        columns = input.columns
-        matrix = input.T.values.tolist()
-
-        columns_fqn = [str(c) for c in (*columns, CHALK_TS_FEATURE)]
-
-        input_times = _normalize_input_times(input_times, len(input))
-
-        matrix.append([a for a in input_times])
-
-        for col_index, column in enumerate(matrix):
-            for row_index, value in enumerate(column):
-                try:
-                    f = Feature.from_root_fqn(columns_fqn[col_index])
-                except FeatureNotFoundException:
-                    # The feature is not in the graph, so passing the value as-is and hoping it's possible
-                    # to json-serialize it
-                    encoded_feature = value
-                else:
-                    encoded_feature = f.converter.from_rich_to_json(value, missing_value_strategy="error")
-                matrix[col_index][row_index] = encoded_feature
-
-        query_input = OfflineQueryInput(
-            columns=columns_fqn,
-            values=matrix,
-        )
+        query_input = _to_offline_query_input(input, input_times)
         request = ComputeResolverOutputRequest(input=query_input, resolver_fqn=resolver)
         response = self._request(
             method="POST",
@@ -829,9 +759,9 @@ class ChalkAPIClientImpl(ChalkClient):
             json=request,
             response=ComputeResolverOutputResponse,
             environment_override=context.environment,
-            use_engine=False,
             exception_cls=ChalkComputeResolverException,
             preview_deployment_id=preview_deployment_id,
+            branch=branch,
         )
         self._raise_if_200_with_errors(response=response, exception_cls=ChalkComputeResolverException)
 
@@ -840,6 +770,7 @@ class ChalkAPIClientImpl(ChalkClient):
                 job_id=response.job_id,
                 context=context,
                 preview_deployment_id=preview_deployment_id,
+                branch=branch,
             )
             if status.is_finished:
                 break
@@ -860,16 +791,17 @@ class ChalkAPIClientImpl(ChalkClient):
         job_id: str,
         context: OfflineQueryContext,
         preview_deployment_id: Optional[str],
+        branch: Optional[BranchId] = None,
     ) -> GetOfflineQueryJobResponse:
         return self._request(
             method="GET",
             uri=f"/v1/compute_resolver_output/{job_id}",
             response=GetOfflineQueryJobResponse,
             json=None,
-            use_engine=False,
             environment_override=context.environment,
             exception_cls=ChalkComputeResolverException,
             preview_deployment_id=preview_deployment_id,
+            branch=branch,
         )
 
     def _create_dataset_request(
@@ -877,6 +809,7 @@ class ChalkAPIClientImpl(ChalkClient):
         request: CreateOfflineQueryJobRequest,
         context: OfflineQueryContext,
         preview_deployment_id: Optional[str],
+        branch: Optional[BranchId] = None,
     ):
         response = self._request(
             method="POST",
@@ -884,9 +817,9 @@ class ChalkAPIClientImpl(ChalkClient):
             json=request,
             response=DatasetResponse,
             environment_override=context.environment,
-            use_engine=False,
             exception_cls=ChalkOfflineQueryException,
             preview_deployment_id=preview_deployment_id,
+            branch=branch,
         )
         return response
 
@@ -895,6 +828,7 @@ class ChalkAPIClientImpl(ChalkClient):
         request: CreateOfflineQueryJobRequest,
         context: OfflineQueryContext,
         preview_deployment_id: Optional[str],
+        branch: Optional[BranchId] = None,
     ):
         response = self._request(
             method="POST",
@@ -902,61 +836,37 @@ class ChalkAPIClientImpl(ChalkClient):
             json=request,
             response=CreateOfflineQueryJobResponse,
             environment_override=context.environment,
-            use_engine=False,
             exception_cls=ChalkOfflineQueryException,
             preview_deployment_id=preview_deployment_id,
+            branch=branch,
         )
         return response
 
     def _get_job_status(
         self,
         job_id: uuid.UUID,
-        context: OfflineQueryContext,
-        preview_deployment_id: Optional[str],
+        environment: Optional[EnvironmentId],
     ) -> GetOfflineQueryJobResponse:
         return self._request(
             method="GET",
             uri=f"/v2/offline_query/{job_id}",
             response=GetOfflineQueryJobResponse,
-            environment_override=context.environment,
-            use_engine=False,
+            environment_override=environment,
             json=None,
             exception_cls=ChalkOfflineQueryException,
-            preview_deployment_id=preview_deployment_id,
+            preview_deployment_id=None,
+            branch=None,
         )
 
-    def _get_dataset(
-        self,
-        dataset_name: str,
-        context: OfflineQueryContext,
-        preview_deployment_id: Optional[str],
-    ) -> DatasetResponse:
+    def _get_dataset(self, dataset_name: str, environment: Optional[EnvironmentId]) -> DatasetResponse:
         return self._request(
             method="GET",
             uri=f"/v3/offline_query/{dataset_name}",
             response=DatasetResponse,
-            environment_override=context.environment,
-            use_engine=False,
+            environment_override=environment,
             json=None,
             exception_cls=ChalkDatasetDownloadException,
             dataset_name=dataset_name,
-            preview_deployment_id=preview_deployment_id,
-        )
-
-    def _get_dataset_urls(
-        self,
-        dataset_name: str,
-        context: OfflineQueryContext,
-        preview_deployment_id: Optional[str],
-    ) -> GetOfflineQueryJobResponse:
-        return self._request(
-            method="GET",
-            uri=f"/v2/lazy_dataset/{dataset_name}",
-            response=GetOfflineQueryJobResponse,
-            environment_override=context.environment,
-            use_engine=False,
-            json=None,
-            exception_cls=ChalkDatasetDownloadException,
-            dataset_name=dataset_name,
-            preview_deployment_id=preview_deployment_id,
+            preview_deployment_id=None,
+            branch=None,
         )
