@@ -1,5 +1,3 @@
-# NOTE: The functions in this file are not tested in chalkpy
-# The integration tests in engine/ validate the behavior of load_dataset
 from __future__ import annotations
 
 import base64
@@ -85,6 +83,9 @@ class DatasetVersion(IntEnum):
     """This format has feature values decoded with their native data types.
     It does not require json decoding client-side"""
 
+    NATIVE_COLUMN_NAMES = 6
+    """This format does not encode column names"""
+
 
 def _parallel_download(uris: List[str], executor: ThreadPoolExecutor, lazy: bool) -> Union[pl.DataFrame, pl.LazyFrame]:
     try:
@@ -123,9 +124,9 @@ def _load_dataset_from_chalk_writer(uris: List[str]) -> pl.DataFrame:
 
 def _decode_column_names(
     column_names: List[str],
+    col_name_decoder: ColNameDecoder | None,
 ) -> Mapping[str, str]:
     ans: Dict[str, str] = {}
-    col_name_decoder = ColNameDecoder()
     for x in column_names:
         if x.startswith("__"):
             if x in ("__id__", ID_FEATURE.fqn):
@@ -138,8 +139,11 @@ def _decode_column_names(
                 ans[x] = OBSERVED_AT_FEATURE.fqn
             # Drop all the other metadata columns
             continue
-        feature_name = col_name_decoder.decode_col_name(x)
-        if any(feature_name.endswith(f".__{x}__") for x in ("oat", "rat", "observed_at", "replaced_observed_at")):
+        if col_name_decoder is None:
+            feature_name = x
+        else:
+            feature_name = col_name_decoder.decode_col_name(x)
+        if any(feature_name.endswith(f".__{y}__") for y in ("oat", "rat", "observed_at", "replaced_observed_at")):
             # Drop the timestamp metadata from individual features
             continue
         ans[x] = feature_name
@@ -193,12 +197,27 @@ def _extract_df_columns(
         DatasetVersion.BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES,
         DatasetVersion.BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES_V2,
         DatasetVersion.NATIVE_DTYPES,
+        DatasetVersion.NATIVE_COLUMN_NAMES,
     ):
-        decoded_col_names = _decode_column_names(df.columns)
+        if version == DatasetVersion.NATIVE_COLUMN_NAMES:
+            col_name_decoder = None
+        else:
+            col_name_decoder = ColNameDecoder()
+        decoded_col_names = _decode_column_names(df.columns, col_name_decoder)
 
         # Select only the columns in decoded_col_names
         df = df.select(list(decoded_col_names.keys()))
         df = df.rename(dict(decoded_col_names))
+        if column_metadata is not None:
+            col_name_set = {x.feature_fqn for x in column_metadata}
+            ordered_cols: list[str] = []
+            for c in df.columns:
+                if c not in col_name_set:
+                    ordered_cols.append(c)
+            for x in column_metadata:
+                if x.feature_fqn not in ordered_cols:
+                    ordered_cols.append(x.feature_fqn)
+            df = df.select(ordered_cols)
 
         # Using an OrderedDict so the order will match the order the user set in the
         # output argument
@@ -306,9 +325,11 @@ def _extract_df_columns(
                 decoded_stmts.append(pl.col(col).dt.with_time_zone("UTC"))
             else:
                 decoded_stmts.append(pl.col(col).apply(_json_decode))
-        elif version == DatasetVersion.NATIVE_DTYPES:
+        elif version in (DatasetVersion.NATIVE_DTYPES, DatasetVersion.NATIVE_COLUMN_NAMES):
             if column_metadata is None:
-                raise ValueError("The columns must be provided if the dataset type is NATIVE_DTYPES")
+                raise ValueError(
+                    "The columns must be provided if the dataset type is NATIVE_DTYPES or NATIVE_COLUMN_NAMES"
+                )
             # We already decoded the column names so matching against the fqn
             if col == CHALK_TS_FEATURE or col == OBSERVED_AT_FEATURE:
                 decoded_stmts.append(pl.col(col).dt.with_time_zone("UTC"))
@@ -380,6 +401,7 @@ def load_dataset(
         DatasetVersion.BIGQUERY_JOB_WITH_B32_ENCODED_COLNAMES_V2,
         DatasetVersion.COMPUTE_RESOLVER_OUTPUT_V1,
         DatasetVersion.NATIVE_DTYPES,
+        DatasetVersion.NATIVE_COLUMN_NAMES,
     ):
         return _load_dataset_bigquery(
             uris,
@@ -400,7 +422,7 @@ class DatasetRevisionImpl(DatasetRevision):
         revision_id: uuid.UUID,
         creator_id: str,
         outputs: List[str],
-        givens_uri: str,
+        givens_uri: str | None,
         status: QueryStatus,
         filters: DatasetFilter,
         num_partitions: int,
@@ -437,23 +459,19 @@ class DatasetRevisionImpl(DatasetRevision):
 
     def get_data_as_polars(
         self,
-        context: Optional[OfflineQueryContext] = None,
-        preview_deployment_id: Optional[str] = None,
         output_id: bool = False,
         output_ts: bool = False,
     ) -> pl.LazyFrame:
-        if context is None:
-            context = OfflineQueryContext()
+        context = OfflineQueryContext()
         _logger.info(f"loading polars LazyFrame for DatasetRevision {self.revision_id}")
         return self._client._await_offline_query_job(
             job_id=self.revision_id,
             outputs=self.outputs,
             lazy=True,
             context=context,
-            preview_deployment_id=preview_deployment_id,
             output_id=output_id,
             output_ts=output_ts,
-        )
+        ).lazy()
 
     @property
     def data_as_pandas(self) -> pd.DataFrame:
@@ -461,13 +479,10 @@ class DatasetRevisionImpl(DatasetRevision):
 
     def get_data_as_pandas(
         self,
-        context: Optional[OfflineQueryContext] = None,
-        preview_deployment_id: Optional[str] = None,
         output_id: bool = False,
         output_ts: bool = False,
     ) -> pd.DataFrame:
-        if context is None:
-            context = OfflineQueryContext()
+        context = OfflineQueryContext()
         _logger.info(f"loading pandas DataFrame for DatasetRevision {self.revision_id}")
         return (
             self._client._await_offline_query_job(
@@ -475,10 +490,10 @@ class DatasetRevisionImpl(DatasetRevision):
                 outputs=self.outputs,
                 lazy=True,
                 context=context,
-                preview_deployment_id=preview_deployment_id,
                 output_id=output_id,
                 output_ts=output_ts,
             )
+            .lazy()
             .collect()
             .to_pandas()
         )
@@ -489,13 +504,10 @@ class DatasetRevisionImpl(DatasetRevision):
 
     def get_data_as_dataframe(
         self,
-        context: Optional[OfflineQueryContext] = None,
-        preview_deployment_id: Optional[str] = None,
         output_id: bool = False,
         output_ts: bool = False,
     ) -> DataFrame:
-        if context is None:
-            context = OfflineQueryContext()
+        context = OfflineQueryContext()
         _logger.info(f"loading Chalk DataFrame for DatasetRevision {self.revision_id}")
         return DataFrame(
             data=self._client._await_offline_query_job(
@@ -503,7 +515,6 @@ class DatasetRevisionImpl(DatasetRevision):
                 outputs=self.outputs,
                 lazy=True,
                 context=context,
-                preview_deployment_id=preview_deployment_id,
                 output_id=output_id,
                 output_ts=output_ts,
             )
@@ -512,19 +523,19 @@ class DatasetRevisionImpl(DatasetRevision):
     def download_data(
         self,
         path: str,
+        output_id: bool = False,
+        output_ts: bool = False,
         num_executors: Optional[int] = None,
-        context: Optional[OfflineQueryContext] = None,
-        preview_deployment_id: Optional[str] = None,
-    ):
-        if context is None:
-            context = OfflineQueryContext()
+    ) -> None:
+        context = OfflineQueryContext()
         urls = self._client._await_offline_query_job(
             job_id=self.revision_id,
             outputs=self.outputs,
             lazy=True,
             context=context,
-            preview_deployment_id=preview_deployment_id,
             urls_only=True,
+            output_id=output_id,
+            output_ts=output_ts,
         )
         if num_executors:
             executor = ThreadPoolExecutor(16)
@@ -598,16 +609,12 @@ class DatasetImpl(Dataset):
 
     def get_data_as_polars(
         self,
-        context: Optional[OfflineQueryContext] = None,
-        preview_deployment_id: Optional[str] = None,
         output_id: bool = False,
         output_ts: bool = False,
     ) -> pl.LazyFrame:
         if len(self._revisions) == 0:
             raise IndexError("No revisions exist for dataset")
         return self._revisions[-1].get_data_as_polars(
-            context=context,
-            preview_deployment_id=preview_deployment_id,
             output_id=output_id,
             output_ts=output_ts,
         )
@@ -618,16 +625,12 @@ class DatasetImpl(Dataset):
 
     def get_data_as_pandas(
         self,
-        context: Optional[OfflineQueryContext] = None,
-        preview_deployment_id: Optional[str] = None,
         output_id: bool = False,
         output_ts: bool = False,
     ) -> pd.DataFrame:
         if len(self._revisions) == 0:
             raise IndexError("No revisions exist for dataset")
         return self._revisions[-1].get_data_as_pandas(
-            context=context,
-            preview_deployment_id=preview_deployment_id,
             output_id=output_id,
             output_ts=output_ts,
         )
@@ -638,16 +641,12 @@ class DatasetImpl(Dataset):
 
     def get_data_as_dataframe(
         self,
-        context: Optional[OfflineQueryContext] = None,
-        preview_deployment_id: Optional[str] = None,
         output_id: bool = False,
         output_ts: bool = False,
     ) -> DataFrame:
         if len(self._revisions) == 0:
             raise IndexError("No revisions exist for dataset")
         return self._revisions[-1].get_data_as_dataframe(
-            context=context,
-            preview_deployment_id=preview_deployment_id,
             output_id=output_id,
             output_ts=output_ts,
         )
@@ -656,11 +655,10 @@ class DatasetImpl(Dataset):
         self,
         path: str,
         num_executors: Optional[int] = 16,
-        context: Optional[OfflineQueryContext] = None,
-        preview_deployment_id: Optional[str] = None,
     ):
         return self._revisions[-1].download_data(
-            path=path, num_executors=num_executors, context=context, preview_deployment_id=preview_deployment_id
+            path=path,
+            num_executors=num_executors,
         )
 
     def __repr__(self) -> str:
