@@ -246,7 +246,7 @@ class FeatureResolutionMeta(BaseModel, frozen=True):
     cache_hit: bool
     """Whether the feature request was satisfied by a cached value."""
 
-    primitive_type: Optional[str]
+    primitive_type: Optional[str] = None
     """
     Primitive type name for the feature, e.g. `str` for `some_feature: str`.
     Returned only if query-level 'include_meta' is True.
@@ -319,6 +319,7 @@ class OnlineQueryRequest(BaseModel):
     staleness: Optional[Mapping[str, str]] = None
     context: Optional[OnlineQueryContext] = None
     include_meta: bool = True
+    explain: bool = False
     skip_online_storage: Optional[bool] = False
     skip_offline_storage: Optional[bool] = False
     skip_metrics_storage: Optional[bool] = False
@@ -372,6 +373,11 @@ class QueryMeta(BaseModel):
     typically have the same hash; changes may be observed over time as we adjust implementation details.
     """
 
+    explain_output: Optional[str] = None
+    """
+    An unstructured string containing diagnostic information about the query execution. Only included if `explain` is True.
+    """
+
 
 class OnlineQueryResponse(BaseModel):
     data: List[FeatureResult]
