@@ -1,14 +1,10 @@
 from typing import Any, List, Optional
 
+from chalk.features._chalkop import Aggregation, op
 from chalk.features._encoding.converter import FeatureConverter
 from chalk.features._encoding.missing_value import MissingValueStrategy
 from chalk.features._encoding.primitive import TPrimitive
-from chalk.features._encoding.serialized_dtype import (
-    BaseSerializedDType,
-    SerializedDType,
-    deserialize_dtype_json,
-    serialize_pyarrow_dtype,
-)
+from chalk.features._encoding.serialized_dtype import deserialize_dtype, serialize_dtype, serialize_pyarrow_dtype
 from chalk.features.dataframe import DataFrame
 from chalk.features.feature_field import Feature, FeatureNotFoundException, feature, has_many, has_one
 from chalk.features.feature_set import Features, FeatureSetBase, is_features_cls
@@ -18,54 +14,10 @@ from chalk.features.feature_wrapper import FeatureWrapper, ensure_feature, unwra
 from chalk.features.filter import Filter, TimeDelta, after, before
 from chalk.features.hooks import after_all, before_all
 from chalk.features.primary import Primary, is_primary
-from chalk.features.resolver import Cron, Resolver, ScheduleOptions, offline, online, sink
+from chalk.features.resolver import Cron, Resolver, ResolverProtocol, offline, online, sink
 from chalk.features.tag import Environments, Tags
 from chalk.utils import MachineType
 
-__all__ = [
-    "Cron",
-    "DataFrame",
-    "Environments",
-    "Feature",
-    "FeatureSetBase",
-    "FeatureTime",
-    "FeatureWrapper",
-    "Features",
-    "Filter",
-    "MachineType",
-    "Primary",
-    "ScheduleOptions",
-    "Tags",
-    "TimeDelta",
-    "after",
-    "after_all",
-    "before",
-    "before_all",
-    "description",
-    "ensure_feature",
-    "feature",
-    "feature_time",
-    "features",
-    "has_many",
-    "has_one",
-    "is_feature_time",
-    "is_primary",
-    "offline",
-    "online",
-    "owner",
-    "sink",
-    "tags",
-    "unwrap_feature",
-    "FeatureNotFoundException",
-    "MissingValueStrategy",
-    "FeatureConverter",
-    "TPrimitive",
-    "BaseSerializedDType",
-    "SerializedDType",
-    "serialize_pyarrow_dtype",
-    "deserialize_dtype_json",
-]
-
 
 def owner(f: Any) -> Optional[str]:
     """Get the owner for a feature, feature class, or resolver.
@@ -83,13 +35,12 @@ def owner(f: Any) -> Optional[str]:
 
     Examples
     --------
-    >>> @features
+    >>> @features(owner="ship")
     ... class RocketShip:
     ...     id: int
-    ...     # Comments above a feature are applied assigned
-    ...     # to the features above which they sit.
-    >>> description(RocketShip.software_version)
-    'Comments above a feature are applied assigned to the features above which they sit.'
+    ...     software_version: str
+    >>> owner(RocketShip.software_version)
+    'ship'
 
     Raises
     ------
@@ -121,11 +72,11 @@ def description(f: Any) -> Optional[str]:
     --------
     >>> @features
     ... class RocketShip:
-    ...     # Comments above a feature are applied assigned
-    ...     # to the features above which they sit.
+    ...     # Comments above a feature become
+    ...     # descriptions for the feature!
     ...     software_version: str
     >>> description(RocketShip.software_version)
-    'Comments above a feature are applied assigned to the features above which they sit.'
+    'Comments above a feature become descriptions for the feature!'
 
     Raises
     ------
@@ -153,7 +104,7 @@ def tags(f: Any) -> Optional[List[str]]:
 
     Returns
     -------
-    str | None
+    list[str] | None
         The tags for a feature, feature class, or resolver, if it exists.
         Note that the tags of a feature could be inherited from the feature class.
 
@@ -188,3 +139,49 @@ def tags(f: Any) -> Optional[List[str]]:
     if isinstance(f, Resolver):
         return f.tags
     raise TypeError(f"Could not determine the tags of '{f}' as it is neither a Feature, Feature Set, or Resolver")
+
+
+__all__ = [
+    "Aggregation",
+    "Cron",
+    "DataFrame",
+    "Environments",
+    "Feature",
+    "FeatureConverter",
+    "FeatureNotFoundException",
+    "FeatureSetBase",
+    "FeatureTime",
+    "FeatureWrapper",
+    "Features",
+    "Filter",
+    "MachineType",
+    "MissingValueStrategy",
+    "Primary",
+    "ResolverProtocol",
+    "deserialize_dtype",
+    "TPrimitive",
+    "Tags",
+    "TimeDelta",
+    "after",
+    "after_all",
+    "before",
+    "before_all",
+    "description",
+    "ensure_feature",
+    "feature",
+    "feature_time",
+    "features",
+    "has_many",
+    "has_one",
+    "is_feature_time",
+    "is_primary",
+    "offline",
+    "online",
+    "op",
+    "owner",
+    "serialize_dtype",
+    "serialize_pyarrow_dtype",
+    "sink",
+    "tags",
+    "unwrap_feature",
+]
