@@ -7,8 +7,8 @@ import json
 import operator
 import os
 import pathlib
-import typing
 import warnings
+from datetime import datetime, timezone
 from typing import (
     TYPE_CHECKING,
     Any,
@@ -16,6 +16,8 @@ from typing import (
     ClassVar,
     Dict,
     List,
+    Literal,
+    Mapping,
     Optional,
     Sequence,
     Set,
@@ -30,11 +32,12 @@ from typing import (
 import pyarrow as pa
 from pydantic import BaseModel
 
+from chalk.features._chalkop import Aggregation
 from chalk.features._encoding.missing_value import MissingValueStrategy
-from chalk.features.dataframe._filters import filter_data_frame
+from chalk.features.dataframe._filters import convert_filters_to_pl_expr, filter_data_frame
 from chalk.features.dataframe._validation import validate_df_schema, validate_nulls
 from chalk.features.feature_field import Feature
-from chalk.features.feature_wrapper import FeatureWrapper, ensure_feature
+from chalk.features.feature_wrapper import FeatureWrapper, ensure_feature, unwrap_feature
 from chalk.features.filter import Filter
 from chalk.utils.collections import ensure_tuple, get_unique_item
 from chalk.utils.missing_dependency import missing_dependency_exception
@@ -44,7 +47,6 @@ if TYPE_CHECKING:
     import polars as pl
 
     from chalk.features.feature_set import Features
-    from chalk.sql import ChalkQueryProtocol, FinalizedChalkQuery, StringChalkQueryProtocol
 
 else:
     try:
@@ -57,16 +59,7 @@ TDataFrame = TypeVar("TDataFrame", bound="DataFrame")
 
 
 class DataFrameMeta(type):
-    # Annotating __getitem__ return a union of DataFrame, query, or string to support resolvers that return sql queries
-    def __getitem__(
-        cls, item: Any
-    ) -> Union[
-        Type[DataFrame],
-        Type[ChalkQueryProtocol],
-        Type[StringChalkQueryProtocol],
-        Type[FinalizedChalkQuery],
-        Type[str],
-    ]:
+    def __getitem__(cls, item: Any) -> Type[DataFrame]:
         from chalk.features.feature_set import Features
 
         # leaving untyped as we type the individual features as their object type
@@ -201,10 +194,158 @@ class DataFrameImpl(metaclass=DataFrameMeta):
     __references_feature_set__: ClassVar[Optional[Type[Features]]] = None
     __pydantic_model__: ClassVar[Optional[Type[BaseModel]]] = None
 
+    def join(
+        self,
+        df: DataFrame,
+        on: Any | List[Any],
+        how: Literal["inner", "left", "outer"] = "left",
+    ) -> DataFrame:
+        on = ensure_tuple(on)
+        p = self._underlying.join(
+            df._underlying,
+            on=[str(t) for t in on],
+            how=how,
+        )
+        return DataFrame(p)
+
+    def group_by(
+        self,
+        group: Mapping[Union[Feature, Any], Union[Feature, Any]],
+        agg: Mapping[Any, Any],
+    ) -> DataFrameImpl:
+        """
+        Parameters
+        ----------
+        group
+            A mapping from the desired column name in the resulting `DataFrame`
+            to the name of the column in the source `DataFrame`.
+        agg
+            A mapping from the desired column name in the resulting `DataFrame`
+            to the aggregation operation to perform on the source `DataFrame`.
+
+        Returns
+        -------
+        DataFrameImpl
+            The `DataFrame` with the specified aggregations applied.
+        """
+        import polars as pl
+
+        from chalk.features.feature_set import FeatureSetBase
+
+        group = [pl.col(str(v)).alias(str(k)) for k, v in group.items()]
+        materialized = self._materialize()
+
+        namespaces = {unwrap_feature(a).namespace for a in agg.keys()}
+        namespace = namespaces.pop() if len(namespaces) == 1 else None
+        timestamp_feature = None if namespace is None else FeatureSetBase.registry[namespace].__chalk_ts__
+
+        now = datetime.now(tz=timezone.utc)
+        cols = []
+        operation: Aggregation
+        for alias, operation in agg.items():
+            # if isinstance(operation, int):
+            #     operation = op.count(FeatureWrapper.len_registry[operation])
+            # c = pl.col(str(operation.col))
+            c = operation.col
+            if len(operation.filters) > 0:
+                c = c.filter(
+                    convert_filters_to_pl_expr(
+                        operation.filters,
+                        self._underlying.schema,
+                        timestamp_feature,
+                        now,
+                    )
+                )
+
+            cols.append(operation.fn(c).alias(str(alias)))
+
+        agged = materialized.lazy().groupby(group).agg(cols).collect()
+
+        return DataFrame(
+            agged,
+            convert_dtypes=self._convert_dtypes,
+            pydantic_model=self._pydantic_model,
+        )
+
+    def with_columns(self, c: Mapping[Any, Any]) -> DataFrame:
+        """Add columns to the `DataFrame`.
+
+        Parameters
+        ----------
+        c
+            A `Mapping` from the desired name of the column
+            in the `DataFrame` to the definition of the new
+            column.
+
+        Examples
+        --------
+        >>> df = DataFrame([
+        ...     User(id=1, first="Sam", last="Wu"),
+        ...     User(id=2, first="Iris", last="Xi")
+        ... ])
+        >>> # Set the fraud score to 0 for all users
+        >>> df.with_columns({User.fraud_score: 0})
+        >>> # Create a full name as the concatenation of first & last
+        >>> df.with_columns({
+        ...     User.full_name: op.concat(User.first, User.last)
+        ... })
+        >>> # Alias a column name
+        >>> df.with_columns({
+        ...     User.first_name: User.first
+        ... })
+
+
+        Returns
+        -------
+        DataFrame
+            A new `DataFrame` with all the existing columns,
+            plus those specified in this function.
+        """
+        import polars as pl
+
+        from chalk.features.feature_set import FeatureSetBase
+
+        namespaces = {unwrap_feature(a).namespace for a in c.keys()}
+        namespace = namespaces.pop() if len(namespaces) == 1 else None
+        timestamp_feature = None if namespace is None else FeatureSetBase.registry[namespace].__chalk_ts__
+
+        now = datetime.now(tz=timezone.utc)
+        cols = []
+        for alias, operation in c.items():
+            # if isinstance(operation, int):
+            #     operation = op.count(FeatureWrapper.len_registry[operation])
+            # c = pl.col(str(operation.col))
+
+            if isinstance(operation, Aggregation):
+                c = operation.col
+                if len(operation.filters) > 0:
+                    c = c.filter(
+                        convert_filters_to_pl_expr(
+                            operation.filters,
+                            self._underlying.schema,
+                            timestamp_feature,
+                            now,
+                        )
+                    )
+
+                cols.append(operation.fn(c).alias(str(alias)))
+            elif isinstance(operation, FeatureWrapper):
+                cols.append(pl.col(str(operation)).alias(str(alias)))
+            else:
+                cols.append(pl.lit(operation).alias(str(alias)))
+
+        agged = self._materialize().lazy().with_columns(cols).collect()
+
+        return DataFrame(
+            agged,
+            convert_dtypes=self._convert_dtypes,
+            pydantic_model=self._pydantic_model,
+        )
+
     def __init__(
         self,
         data: Union[
-            Dict[Union[str, Feature, FeatureWrapper, Any], Sequence[Any]],
+            Dict[Union[str, Feature, Any], Sequence[Any]],
             Sequence[Union[Features, Any]],
             pl.DataFrame,
             pl.LazyFrame,
@@ -427,7 +568,7 @@ class DataFrameImpl(metaclass=DataFrameMeta):
         return df
 
     @classmethod
-    def _get_storage_options(cls) -> typing.Optional[Dict[str, Any]]:
+    def _get_storage_options(cls) -> Optional[Dict[str, Any]]:
         if os.getenv("GCP_INTEGRATION_CREDENTIALS_B64") is not None:
             try:
                 from google.oauth2 import service_account
