@@ -87,7 +87,7 @@ def validate_df_schema(underlying: Union[pl.DataFrame, pl.LazyFrame]):
             # If all values are null, then replace with a null series of the correct datatype
             # It's quite possible that the original column will have an incorrect dtype if all values are null,
             # since there was no data to infer the correct dtype from
-            underlying = underlying.with_column(
+            underlying = underlying.with_columns(
                 _generate_empty_series_for_dtype(root_fqn, expected_dtype, len(underlying))
             )
         elif _polars_dtype_contains_struct(expected_dtype):
@@ -125,10 +125,10 @@ def validate_df_schema(underlying: Union[pl.DataFrame, pl.LazyFrame]):
             else:
                 col = pl.col(root_fqn).cast(expected_dtype)
             try:
-                underlying = underlying.with_column(col)
+                underlying = underlying.with_columns(col)
             except pl.ComputeError as e:
                 raise TypeError(
-                    f"Values for feature `{root_fqn}` could not be converted to dtype `{expected_dtype.string_repr()}`. Found type {actual_dtype}, instead."
+                    f"Values for feature `{root_fqn}` could not be converted to dtype `{expected_dtype}`. Found type {actual_dtype}, instead."
                 ) from e
     return underlying
 
