@@ -1,6 +1,7 @@
 from __future__ import annotations
 
-from typing import Any, Callable, ClassVar, Dict, Iterator, List, Optional, Tuple, Type, cast
+from datetime import timedelta
+from typing import Any, Callable, ClassVar, Dict, Iterator, List, Optional, Tuple, Type, Union, cast
 
 from typing_extensions import TypeGuard
 
@@ -67,6 +68,9 @@ class FeaturesMeta(type):
         cls = cast(Type[Features], cls)
         return f"Features[{', '.join(str(f) for f in cls.features)}]"
 
+    def __call__(cls, *args: object, **kwargs: object):
+        raise RuntimeError("Instances features cls should never be directly created. Instead, use Features[User.id]")
+
     @property
     def namespace(cls):
         cls = cast(Type[Features], cls)
@@ -83,13 +87,10 @@ class FeaturesImpl(metaclass=FeaturesMeta):
     typing.
     """
 
-    def __new__(cls, *args: object, **kwargs: object):
-        raise RuntimeError("Instances features cls should never be directly created. Instead, use Features[User.id]")
-
     # Internally, the Features class is instantiated when results come through, and
     # results are bound to instances of this class via attributes
     __chalk_etl_offline_to_online__: ClassVar[bool]
-    __chalk_max_staleness__: ClassVar[Optional[Duration]]
+    __chalk_max_staleness__: ClassVar[timedelta]
     __chalk_namespace__: ClassVar[str]
     __chalk_primary__: ClassVar[Optional[Feature]]  # The primary key feature
     __chalk_owner__: ClassVar[Optional[str]]
