@@ -4,7 +4,8 @@ import contextlib
 import csv
 import io
 import logging
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Mapping, Optional, Union
+import os
+from typing import Any, Callable, Dict, List, Optional, TYPE_CHECKING, Union
 
 import pyarrow as pa
 import pyarrow.csv
@@ -91,7 +92,7 @@ class PostgreSQLSourceImpl(BaseSQLSource, TableIngestMixIn, SQLSourceWithTableIn
         default = dict(
             pool_size=20,
             max_overflow=60,
-            pool_pre_ping=True,
+            pool_pre_ping=os.getenv('USE_CLIENT_POOL_PRE_PING', 'true') == 'true',
             # Trying to fix mysterious dead connection issue
             connect_args={
                 "keepalives": 1,
