@@ -28,6 +28,7 @@ from chalk.streams.base import StreamSource
 from chalk.streams.types import StreamResolverSignature
 from chalk.utils.duration import parse_chalk_duration
 from chalk.utils.missing_dependency import missing_dependency_exception
+from chalk.utils.string import to_camel_case
 
 if TYPE_CHECKING:
     from sqlglot.expressions import Expression
@@ -505,7 +506,7 @@ def _parse_glot(glot_result: GlotResult, path: str, sources: Iterable[BaseSQLSou
                 source = possible_source
     if not source:
         parse_result.errors.append(
-            ResolverError(display=f"Source {source_name} not found", path=path, parameter=source_name)
+            ResolverError(display=f"Source '{source_name}' not found", path=path, parameter=source_name)
         )
     parse_result.source = source
 
@@ -514,6 +515,16 @@ def _parse_glot(glot_result: GlotResult, path: str, sources: Iterable[BaseSQLSou
 
     # get resolver fields: which columns selected will match to which chalk feature?
     namespace = parse_result.comment_dict.resolves
+    if namespace not in FeatureSetBase.registry:
+        camelcase_namespace = to_camel_case(namespace)
+        if camelcase_namespace in FeatureSetBase.registry:
+            namespace = camelcase_namespace
+        else:
+            parse_result.errors.append(
+                ResolverError(
+                    display=f"No @features class with the name '{namespace}'", path=path, parameter=source_name
+                )
+            )
     parse_result.namespace = namespace
     fields = {}  # sql string -> output feature string
 
@@ -568,6 +579,7 @@ def _get_stream_resolver(
             state=None,
             sql_query=sql_query,
             owner=parsed.comment_dict.owner,
+            parse=None,
         )
     except Exception as e:
         result.errors.append(
