@@ -4,6 +4,7 @@ import contextlib
 import functools
 import inspect
 import json
+import logging
 import os
 import os.path
 import warnings
@@ -67,13 +68,10 @@ def _force_set_str(x: Optional[List[Any]]) -> Set[str]:
     return set() if x is None else set(map(str, x))
 
 
-class UnsupportedDtypesForEfficientExecutionError(ValueError):
-    def __init__(
-        self, unsupported_columns_and_dtypes: Sequence[Tuple[str, str]], supported_dtypes: Sequence[str]
-    ) -> None:
-        self.unsupported_columns_and_dtypes = unsupported_columns_and_dtypes
-        self.supported_dtypes = supported_dtypes
-        super().__init__()
+class UnsupportedEfficientExecutionError(ValueError):
+    def __init__(self, msg: str, log_level: int) -> None:
+        super().__init__(msg)
+        self.log_level = log_level
 
 
 def validate_dtypes_for_efficient_execution(stmt: Select, supported_types: Sequence[Type[TypeEngine]]):
@@ -85,9 +83,18 @@ def validate_dtypes_for_efficient_execution(stmt: Select, supported_types: Seque
         if x["type"].__class__ not in supported_types
     ]
     if len(unsupported_columns) > 0:
-        raise UnsupportedDtypesForEfficientExecutionError(
-            unsupported_columns_and_dtypes=[(x["name"], x["type"].__class__.__name__) for x in unsupported_columns],
-            supported_dtypes=[x.__name__ for x in supported_types],
+        unsupported_columns_and_dtypes = [(x["name"], x["type"].__class__.__name__) for x in unsupported_columns]
+        supported_dtypes = [x.__name__ for x in supported_types]
+        formatted_supported = ", ".join(supported_dtypes)
+        formatted_unsupported = ", ".join([f"{name}({dtype})" for (name, dtype) in unsupported_columns_and_dtypes])
+        raise UnsupportedEfficientExecutionError(
+            (
+                "The SQL statement will be executed into SQLAlchemy objects, as the SQL query returns columns "
+                "that cannot be loaded directly into a PyArrow table. For better performance, use only "
+                f"the following types in your SQLAlchemy model: {formatted_supported}."
+                f"The columns that contain unsupported types are: {formatted_unsupported}."
+            ),
+            log_level=logging.INFO,
         )
 
 
@@ -147,7 +154,7 @@ class BaseSQLSource(BaseSQLSourceProtocol):
         try:
             fqn = get_logging_context().get("labels", {}).get("resolver_fqn")
             if fqn is not None:
-                sql_string = self._resolver_and_sqlfile_to_sqlstring.get((fqn, path))
+                sql_string = self._resolver_and_sqlfile_to_sqlstring.get((fqn, str(path)))
         except Exception:
             pass
 
@@ -170,7 +177,7 @@ class BaseSQLSource(BaseSQLSourceProtocol):
             raise FileNotFoundError(f"No such file: '{str(path)}'")
 
         if uncached and fqn is not None:
-            self._resolver_and_sqlfile_to_sqlstring[(fqn, path)] = sql_string
+            self._resolver_and_sqlfile_to_sqlstring[(fqn, str(path))] = sql_string
 
         return self.query_string(
             query=sql_string,
@@ -221,39 +228,19 @@ class BaseSQLSource(BaseSQLSourceProtocol):
     ) -> pa.Table:
         from sqlalchemy.engine import Connection
 
-        if env_var_bool("CHALK_FORCE_SQLALCHEMY_QUERY_EXECUTION"):
+        try:
+            return self.execute_query_efficient(finalized_query, columns_to_converters, connection)
+        except NotImplementedError:
+            _logger.info(
+                (
+                    "The SQL statement will be executed into SQLAlchemy objects, as the database backend does "
+                    "not support a more efficient execution mechanism."
+                )
+            )
             pass
-            # _logger.debug(
-            #     (
-            #         f"The SQL statement will be executed into SQLAlchemy objects, as the environment "
-            #         "variable 'CHALK_FORCE_SQLALCHEMY_QUERY_EXECUTION' is set. Unset this variable to execute "
-            #         "the query directly into a PyArrow table."
-            #     )
-            # )
-        else:
-            try:
-                return self.execute_query_efficient(finalized_query, columns_to_converters, connection)
-            except NotImplementedError:
-                _logger.info(
-                    (
-                        "The SQL statement will be executed into SQLAlchemy objects, as the database backend does "
-                        "not support a more efficient execution mechanism."
-                    )
-                )
-                pass
-            except UnsupportedDtypesForEfficientExecutionError as e:
-                formatted_supported = ", ".join(e.supported_dtypes)
-                formatted_unsupported = ", ".join(
-                    [f"{name}({dtype})" for (name, dtype) in e.unsupported_columns_and_dtypes]
-                )
-                _logger.info(
-                    (
-                        "The SQL statement will be executed into SQLAlchemy objects, as the SQL query returns columns "
-                        "that cannot be loaded directly into a PyArrow table. For better performance, use only "
-                        f"the following types in your SQLAlchemy model: {formatted_supported}."
-                        f"The columns that contain unsupported types are: {formatted_unsupported}."
-                    )
-                )
+        except UnsupportedEfficientExecutionError as e:
+            log_level = e.log_level
+            _logger.log(log_level, str(e))
 
         with self.get_engine().connect() if connection is None else contextlib.nullcontext(connection) as cnx:
             assert isinstance(cnx, Connection)
@@ -309,8 +296,26 @@ class BaseSQLSource(BaseSQLSourceProtocol):
             assert isinstance(extra_args, dict), "ENGINE_ARGUMENTS must be a JSON object"
             return extra_args
 
+    def get_engine(self, additional_engine_args: Optional[Dict[str, Any]] = None) -> Engine:
+        if additional_engine_args is None:
+            return self._get_engine_default()
+        else:
+            return self._get_engine_with_args(additional_engine_args)
+
+    def _get_engine_with_args(self, additional_engine_args: Dict[str, Any]) -> Engine:
+        from sqlalchemy.engine import create_engine
+
+        args = self.engine_args()
+        args.update(additional_engine_args)
+
+        return create_engine(
+            url=self.local_engine_url(),
+            echo=CHALK_QUERY_LOGGING,
+            **args,
+        )
+
     @functools.lru_cache(None)  # pyright: ignore[reportIncompatibleMethodOverride]
-    def get_engine(self) -> Engine:
+    def _get_engine_default(self) -> Engine:
         from sqlalchemy.engine import create_engine
 
         return create_engine(
@@ -360,10 +365,13 @@ def _extract_features(feature_set: Features, prefix: str, targets: List[Label],
         elif isinstance(feature_value, (sqlalchemy.sql.functions.GenericFunction, InstrumentedAttribute)):
             features[root_fqn] = Feature.from_root_fqn(root_fqn)
             targets.append(feature_value.label(root_fqn))
+        elif isinstance(feature_value, sqlalchemy.sql.schema.Column):
+            features[root_fqn] = Feature.from_root_fqn(root_fqn)
+            targets.append(feature_value.label(root_fqn))
         else:
             raise TypeError(
                 (
-                    f"Feature '{root_fqn}' has an unsupported value for SQL queries. "
+                    f"Feature '{root_fqn}' has an unsupported value of type '{type(feature_value)}' for SQL queries. "
                     "All values must be a column, a column name string, a nested Features class, or an SQLAlchemy function"
                 )
             )
