@@ -6,6 +6,7 @@ from typing import (
     Any,
     Callable,
     Dict,
+    Iterator,
     List,
     Literal,
     Mapping,
@@ -27,7 +28,7 @@ from chalk.utils.duration import Duration
 
 if TYPE_CHECKING:
     import sqlalchemy
-    from sqlalchemy.engine import URL, Engine
+    from sqlalchemy.engine import URL
     from sqlalchemy.orm import Session
 
     from chalk.sql.finalized_query import FinalizedChalkQuery
@@ -486,12 +487,17 @@ class BaseSQLSourceProtocol(Protocol):
     def local_engine_url(self) -> URL:
         ...
 
-    def engine_args(self) -> Mapping[str, Any]:
+    def engine_args(self) -> Dict[str, Any]:
         return {}
 
-    def get_engine(self) -> sqlalchemy.engine.Engine:
+    def get_engine(self, additional_engine_args: Optional[Dict[str, Any]] = None) -> sqlalchemy.engine.Engine:
         """Get an SQLAlchemy Engine. The engine will be created and cached on the first call of this method.
 
+        Parameters
+        ----------
+        additional_engine_args
+            Optional additional engine arguments, merged with the default arguments.
+
         Returns
         -------
         sqlalchemy.engine.Engine
@@ -506,6 +512,22 @@ class BaseSQLSourceProtocol(Protocol):
     def raw_session(self) -> Session:
         ...
 
+    def execute_query_to_pyarrow_tables(
+        self,
+        finalized_query: FinalizedChalkQuery,
+        columns_to_converters: Callable[[List[str]], Dict[str, FeatureConverter]],
+        connection: Optional[Any] = None,
+    ) -> Iterator[pa.Table]:
+        ...
+
+    def execute_query_to_batches(
+        self,
+        finalized_query: FinalizedChalkQuery,
+        columns_to_converters: Callable[[List[str]], Dict[str, FeatureConverter]],
+        connection: Optional[Any] = None,
+    ) -> List[pa.Table]:
+        ...
+
     def execute_query(
         self,
         finalized_query: FinalizedChalkQuery,
