@@ -9,5 +9,5 @@ class FileSource(BaseModel, StreamSource, frozen=True):
     path: str
     key_separator: str = "|"
 
-    def config_to_json(self) -> Any:
+    def _config_to_json(self) -> Any:
         return self.json()
