@@ -3,9 +3,10 @@ import os
 import pathlib
 import typing
 import warnings
+from collections import defaultdict
 from typing import Dict, List, NamedTuple, Optional, Tuple, Type, Union
 
-from typing_extensions import Annotated, TypeAlias, get_args, get_origin
+from typing_extensions import Annotated, get_args, get_origin
 
 from chalk.features import Feature, Features, FeatureSetBase
 from chalk.importer import get_directory_root, import_all_python_files_from_dir
@@ -129,14 +130,17 @@ class StubGenerator:
             else:
                 # Make a best-effort to figure out imports.
                 annotation = self._register_imports_for_annotation(f.typ.parsed_annotation)
-                annotations[f.attribute_name] = ParsedAnnotation(
-                    annotation,
-                    include_in_protocol_cls=not f.is_windowed_pseudofeature and not f.is_autogenerated,
-                    # If it's a features class (i.e. a has-one), not specifying
-                    # the type in the protocol class so the type checker can
-                    # still match feature classes that have a circular reference
-                    protocol_annotation="..." if f.typ.is_features_cls else annotation,
-                )
+                include_in_proto_cls = not f.is_windowed_pseudofeature and not f.is_autogenerated
+                if include_in_proto_cls:
+                    # TODO do not skip the annotations that do not appear in the protocol class
+                    annotations[f.attribute_name] = ParsedAnnotation(
+                        annotation,
+                        include_in_protocol_cls=include_in_proto_cls,
+                        # If it's a features class (i.e. a has-one), not specifying
+                        # the type in the protocol class so the type checker can
+                        # still match feature classes that have a circular reference
+                        protocol_annotation="Any" if f.typ.is_features_cls else annotation,
+                    )
         if features_cls.__name__ in self._parsed_feature_classes:
             raise ValueError(
                 f"Unable to generate stubs due to multiple features sets called {features_cls.__name__}. Feature set names must be unique. Please rename one of these feature sets."
@@ -148,6 +152,7 @@ class StubGenerator:
 
     def generate_features_decorator_stub_file(self):
         self._add_import("typing.Optional", "Optional")
+        self._add_import("typing.Any", "Any")
         self._add_import("typing.Protocol", "Protocol")
         self._add_import("typing.Type", "Type")
         self._add_import("typing.overload", "overload")
@@ -186,18 +191,63 @@ class StubGenerator:
         # Class 3: The protocol class. Since the @features decorator is passed the end user's class definition, and returns the
         # Chalk-ified version of it, we cannot "import" the user's class definition. So, we define a protocol class that contains the
         # same instance attributes. The type checker can match the user's class against the protocol.
-        #
+
+        # Since we effectively duck type with Protocols, different feature sets with all the same attributes
+        # and types cannot be distinguished
+        # So, we will keep just one feature set for a given set of attributes and annotations
+        attributes_and_annotations_to_feature_set_names: dict[frozenset[tuple[str, str]], list[str]] = defaultdict(list)
+        for x_name, x in self._parsed_feature_classes.items():
+            attributes_and_annotations_to_feature_set_names[
+                frozenset((k, v.protocol_annotation) for (k, v) in x.annotations.items())
+            ].append(x_name)
+        feature_set_names_to_keep: set[str] = set()
+        for feature_set_names in attributes_and_annotations_to_feature_set_names.values():
+            # Sorting for stability
+            feature_set_names_to_keep.add(sorted(feature_set_names)[0])
+        self._parsed_feature_classes = {
+            k: v for (k, v) in self._parsed_feature_classes.items() if k in feature_set_names_to_keep
+        }
+
+        # Update the base classes
+        # Feature sets that are supersets of other feature classes must inherit from the base classes, due to how
+        # @overload requires return and parameter types of the less specific classes to be supertypes
+        parsed_features_class_name_to_bases: dict[str, list[str]] = {}
+        for x_name, x in self._parsed_feature_classes.items():
+            common_base_class_names: set[str] = set()
+            for y_name, y in self._parsed_feature_classes.items():
+                if x is not y:
+                    if all(
+                        x.annotations.get(attribute_name) == annotation
+                        for attribute_name, annotation in y.annotations.items()
+                    ):
+                        # If all of y's annotations are a subset of x's, it's a possible base class
+                        common_base_class_names.add(y_name)
+            # Sorting the base classes in reverse order so the most specific will go first
+            common_base_classes_sorted = sorted(
+                common_base_class_names,
+                key=lambda y_name: (len(self._parsed_feature_classes[y_name].annotations), y_name),
+                reverse=True,
+            )
+            parsed_features_class_name_to_bases[x_name] = [*common_base_classes_sorted, "Features"]
+
         # An @overload is used to match each protocol class (class #3) to the typestub version of the class (class #2)
-        # Since the first matching protocol will be used, the feature sets with the _most_ features should appear first
+        # First, define all metaclasses, then the classes, then the protocols, most general to most specific
+        # However, when defining the @overloads, go from most specific to most general, since the first matching @overload
+        # is used
         # Also including the class name as part of the sort key for stability
         parsed_feature_classes = sorted(
-            self._parsed_feature_classes.items(), key=lambda x: (len(x[1].annotations), x[0]), reverse=True
+            self._parsed_feature_classes.items(),
+            key=lambda x: (len(x[1].annotations), x[0]),
         )
+
         for feature_cls_name, features_cls in parsed_feature_classes:
             # First generate the metaclass definition
+            metaclasses_bases = ", ".join(
+                (*(f"{base_class}Meta" for base_class in parsed_features_class_name_to_bases[feature_cls_name]), "type")
+            )
             lines.append(
                 f"""\
-class {feature_cls_name}Metaclass(FeaturesMeta):"""
+class {feature_cls_name}Meta({metaclasses_bases}):"""
             )
             for attribute, annotation in features_cls.annotations.items():
                 lines.append(
@@ -212,10 +262,13 @@ class {feature_cls_name}Metaclass(FeaturesMeta):"""
     ...
 """
                 )
+
+        for feature_cls_name, features_cls in parsed_feature_classes:
             # Next, generate the class definition
+            bases = ", ".join(parsed_features_class_name_to_bases[feature_cls_name])
             lines.append(
                 f"""\
-class {feature_cls_name}(Features, metaclass={feature_cls_name}Metaclass):"""
+class {feature_cls_name}({bases}, metaclass={feature_cls_name}Meta):"""
             )
             lines.append(
                 f"""\
@@ -242,6 +295,7 @@ class {feature_cls_name}(Features, metaclass={feature_cls_name}Metaclass):"""
         ..."""
                 )
             lines.append("")
+        for feature_cls_name, features_cls in parsed_feature_classes:
             # Finally, generate the protocol class representing the features class that is being transformed
             lines.append(
                 f"""\
@@ -267,7 +321,15 @@ class {feature_cls_name}Protocol(Protocol):"""
         # instances of the classes defined above, not the classes defined in the user's module
         # This allows us to put all type stubs in one file, under the chalk/features/feature namespace,
         # rather than having to create a type stub for each user's feature class.
-        for features_cls_name, features_cls in parsed_feature_classes:
+        proto_sorted_feature_classes = sorted(
+            self._parsed_feature_classes.items(),
+            key=lambda x: (
+                sum(1 for annotation in x[1].annotations.values() if annotation.include_in_protocol_cls),
+                x[0],
+            ),
+            reverse=True,
+        )
+        for features_cls_name, features_cls in proto_sorted_feature_classes:
             # For each features class, adding two overloads. The first overload transforms the definition
             # from the user's code into our stub. However, since we are technically importing the version that
             # has already been transformed with @features, we need to annotate how to handle a class that
@@ -298,7 +360,7 @@ def features(
 
 class __stubgen__features_proto(Protocol):"""
         )
-        for features_cls_name, features_cls in parsed_feature_classes:
+        for features_cls_name, features_cls in proto_sorted_feature_classes:
             overload = (
                 "@overload" if len(parsed_feature_classes) > 1 else ""
             )  # It is incorrect to use @overload if there is just one definition
